/*
 * Copyright (c) 2007 - 2009 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA Corporation is strictly prohibited.
 */

/** @file
 * <b>NVIDIA Tegra BSP DirectShow API Extensions:
 *         Encode/Capture Filter API</b>
 *
 * @brief Declares CLSIDs used to create capture and encode filters.
 */

#ifndef INCLUDED_NVDMO_H
#define INCLUDED_NVDMO_H

#include "nvcommon.h"

/** @defgroup   nvdshow_encode_group Encode/Capture Filter API
 *
 * This file contains all definitions required to use filters related to
 * the capture and encode chain.
 *
 * @par Capture Filters:
 * - NVIDIA Video Capture Filter (libnvvideo_capture.dll)
 * - NVIDIA Audio Capture Filter (libnvaudio_capture.dll)
 *
 * @par Encoder Filters (DirectShow):
 * - NVIDIA Video Encoder Filter (libnvvideo_encoder.dll)
 * - NVIDIA Audio Encoder Filter (libnvaudio_encoder.dll)
 *
 * @par Encoder Filters (DMO):
 * - NVIDIA Video Encoder DMO (libnvvideo_encoder_dmo.dll)
 * - NVIDIA Audio Encoder DMO (libnvaudio_encoder_dmo.dll)
 *
 * @par File Writer Filters:
 * - NVIDIA Mux File Writer Filter (libnv_mux_file_writer.dll)
 *
 *   @note DMO filters are available on WM6 only or when
 *   \c NV_INCLUDE_ENCODER_DMO environment variable is set. Encoder
 *   Filters (DirectShow) are DirectShow filters and preferred over
 *   DMO filters on all OSs.
 *
 * @ingroup nvdshow_extension
 * @{
 */

/** @name Filter CLSIDs
 * These are the CLSIDs used to create capture and encode filters.
 */
/*@{*/

/**
 * NVIDIA Audio Encoder Filter (DirectShow)
 *
 * @par Supports:
 * AAC-LC, AAC+, E-AAC+, AMR-NB, AMR-WB
 *
 * @par GUID:
 * {c98a8f9b-1074-4671-b5a9-58c8f42179c6}
 *
 * The following list shows the output media subtype (minor type) supported
 * by thier respective NVIDIA media type (major type).
 *
 * @note For all audio/speech codecs, the output type
 *   major type is \c MEDIATYPE_NvAudio.
 * 
 *  - MEDIASUBTYPE_AMRAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_AWBAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_AACAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_AACPLUSAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_EAACPLUSAudio (minor type), MEDIATYPE_NvAudio (major type)
 *
 * The following shows the input type supported (minor type) supported by
 * its NVIDIA media type (major type):
 * - MEDIASUBTYPE_PCM (minor type), MEDIATYPE_NvAudioBuffered (major type)
 */
DEFINE_GUID(CLSID_NvAudioEncoder,
0xc98a8f9b, 0x1074, 0x4671, 0xb5, 0xa9, 0x58, 0xc8, 0xf4, 0x21, 0x79, 0xc6);

/**
 * NVIDIA Audio Encoder DMO, which is included in Tegra BSP only
 * for WM6 or when \c NV_INCLUDE_ENCODER_DMO environment variable is
 * set. NVIDIA Audio Encoder Filter (DirectShow) must be preferred
 * over this filter, because it is a DirectShow filter that provides all
 * features of this filter. The DirectShow filter is available on all
 * OSs.
 *
 * @par Supports:
 * AAC-LC, AAC+, E-AAC+, AMR-NB, AMR-WB 
 * 
 * @par GUID:
 * {db6e4382-4995-490e-ba43-60383fe625d5}
 */
DEFINE_GUID(CLSID_NvAudioEncoderDMO,
0xdb6e4382, 0x4995, 0x490e, 0xba, 0x43, 0x60, 0x38, 0x3f, 0xe6, 0x25, 0xd5);

/**
 * NVIDIA Null Video Encoder DMO is included in only when the
 * \c NV_USE_ENCODING_IN_CAMERA environment variable is set.
 *
 * @par GUID:
 * {11e69b15-45f0-4c80-b9a7-1f56fe8e572b}
 */
DEFINE_GUID(CLSID_NvNullVideoEncoderDMO,
0x11e69b15, 0x45f0, 0x4c80, 0xb9, 0xa7, 0x1f, 0x56, 0xfe, 0x8e, 0x57, 0x2b);

/**
 * NVIDIA Mux File Writer Filter
 *
 * @par Supports:
 * - MUX of encoded H.264, H.263, MPEG-4 video data
 * - Audio AAC-LC, AAC+, E-AAC+, AMR-NB, AMR-WB audio data
 * - File writing into 3GP file format
 * - Writing JPEG stream to file
 * @note Still image dimension cannot be greater than the capture dimension
 *   when creating both still image and capture pin at the same time.
 *
 * @par GUID:
 * {74f08d23-78e3-4f2a-bd84-532038d9cede}
 *
 */
DEFINE_GUID(CLSID_Nv3GP2MuxFilter,
0x74f08d23, 0x78e3, 0x4f2a, 0xbd, 0x84, 0x53, 0x20, 0x38, 0xd9, 0xce, 0xde);

/**
 * NVIDIA Video Encoder Filter (DirectShow)
 *
 * @par Supports:
 * H.264, H.263, and MPEG-4 video encoding formats
 *
 * @par GUID:
 * {E5D912CD-FE4D-4ee2-A9E1-C3B7984029B0}
 *
 * The following list shows the output media subtype (minor type) supported
 * by thier respective media type (major type).
 *
 * @note For all video/still-image output types, the major types
 *   \c MEDIATYPE_NvVideo as well as \c MEDIATYPE_Video are supported.
 *
 *  - MEDIASUBTYPE_H264Video (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_MPEG4Video (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_H263Video (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_NvJPEG (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_H264Video (minor type), MEDIATYPE_Video (major type)
 *  - MEDIASUBTYPE_MPEG4Video (minor type), MEDIATYPE_Video (major type)
 *  - MEDIASUBTYPE_H263Video (minor type), MEDIATYPE_Video (major type)
 *  - MEDIASUBTYPE_IJPG (minor type), MEDIATYPE_Video (major type)
 *
 * The video encoder's frame dimension is the same as that of the frame
 * dimension set on the video capture filter. The frame length (sample size)
 * is decided based upon frame dimension and bit-count at connection time.
 *
 * The following list shows the input type supported (minor type) supported by
 * their respective media type (major type):
 *
 *  - MEDIASUBTYPE_NvMMBuffer (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_VideoBuffered (major type)
 *  - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_VideoBuffered (major type)
 */
DEFINE_GUID(CLSID_NvVideoEncoder,
0xe5d912cd, 0xfe4d, 0x4ee2, 0xa9, 0xe1, 0xc3, 0xb7, 0x98, 0x40, 0x29, 0xb0);

/**
 * NVIDIA Video Encoder DMO is included only for WM6 or when
 * \c NV_INCLUDE_ENCODER_DMO environment variable is set.
 * NVIDIA Video Encoder Filter (DirectShow) must be preferred over this
 * filter, because it is a DirectShow filter that provides all features of
 * this filter. The DirectShow filter is available on all OSs.
 *
 * @par Supports:
 * H.264, H.263 and MPEG-4 video encoding formats
 *
 * @par GUID:
 * {bc72063c-6a8c-4118-a9a9-d8ca54a56d1f}
 */
DEFINE_GUID(CLSID_NvVideoEncoderDMO,
0xbc72063c, 0x6a8c, 0x4118, 0xa9, 0xa9, 0xd8, 0xca, 0x54, 0xa5, 0x6d, 0x1f);

/**
 * NVIDIA Video Capture Filter
 *
 * The video capture filter is a 3-pin filter. This filter exposes the
 * preview pin, capture pin, and still pin.
 *
 * Supported media types (major and minor types) on the preview pin:
 * - MEDIASUBTYPE_NvYUV420 (minor type), MEDIATYPE_NvRendererVideo (major type)
 * - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_Video (major type)
 * - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_Video (major type)
 *
 * Supported media types (major and minor types) on the capture pin:
 * - MEDIASUBTYPE_NvMMBuffer (minor type), MEDIATYPE_NvVideo (major type)
 * - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_VideoBuffered (major type)
 * - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_VideoBuffered (major type)
 *
 * Supported media type (major and minor type) on the still pin:
 * - MEDIASUBTYPE_NvMMBuffer (minor type), MEDIATYPE_NvVideo (major type)
 * - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_VideoBuffered (major type)
 * - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_VideoBuffered (major type)
 * - MEDIASUBTYPE_Bayer (minor type), MEDIATYPE_NvVideo (major type)
 *
 * @par Setting Frame Dimension
 *
 * The frame dimension can be set for all 3 pins separately using the
 * <b>IAMStreamConfig::SetFormat</b> method. The frame dimension on a pin must
 * be set before that pin is rendered (connected to another filter). If
 * not set, the capture filter will use a default frame dimension.
 *
 * If all 3 pins, preview, capture and still are connected at the same time
 * then still pin's dimension cannot be more than the capture pin's dimension.
 * This limitation does not apply when either the capture pin or the still
 * pin remains unconnected.
 *
 * @code
 *
 * VIDEOINFOHEADER vih = {0};
 * vih.AvgTimePerFrame = UNITS / 30;        // Frame rate
 * vih.bmiHeader.biWidth = 720;             // Frame width
 * vih.bmiHeader.biHeight = 480;            // Frame height
 *
 * AM_MEDIA_TYPE amt = {0};
 * amt.formattype = FORMAT_VideoInfo;
 * amt.pbFormat = (BYTE *)&vih;
 * amt.cbFormat = sizeof(VIDEOINFOHEADER);
 *
 * IAMStreamConfig *pStreamConfig = NULL;
 *
 * // Code to enumerate pins, determine pin category and QueryInterface for
 * // IAMStreamConfig from pin's IPin interface.
 *
 * pStreamConfig->SetFormat(&amt);
 *
 * @endcode
 *
 * @par See also
 * For more information, see MSDN documentation on @b IAMStreamConfig at:
 * <a href="http://msdn.microsoft.com/en-us/library/aa915620.aspx" 
 * target="_blank">http://msdn.microsoft.com/en-us/library/aa915620.aspx</a>
 * 
 * @par GUID:
 * {cdc2b6ec-87c0-489d-be56-f522d8406c4b}
 */
DEFINE_GUID(CLSID_NvVideoCapture,
0xCDC2B6EC, 0x87C0, 0x489D, 0xBE, 0x56, 0xF5, 0x22, 0xD8, 0x40, 0x6C, 0x4B);

/**
 * NVIDIA Audio Capture Filter
 *
 * The audio capture filter exposes 1 output pin. Supported major type and
 * minor type:
 * - MEDIASUBTYPE_PCM (minor type), MEDIATYPE_AudioBuffered (major type)
 *
 * @par GUID:
 * {b8300c7c-3390-43ab-acc0-29e58e7e390e}
 */
DEFINE_GUID(CLSID_NvAudioCapture,
0xb8300c7c, 0x3390, 0x43ab, 0xac, 0xc0, 0x29, 0xe5, 0x8e, 0x7e, 0x39, 0xe);

/**
 * NVIDIA File Source Filter is used to read video and audio
 * data and feed it to the encoders. NVIDIA File Source Filter is
 * included only for WM6 or when the \c NV_INCLUDE_ENCODER_DMO
 * environment variable is set.
 *
 * @par GUID:
 * {f8771ad2-2b0d-4654-9d29-faff2552af14}
 */
DEFINE_GUID(CLSID_NvFileSource,
0xf8771ad2, 0x2b0d, 0x4654, 0x9d, 0x29, 0xfa, 0xff, 0x25, 0x52, 0xaf, 0x14);

/**
 * NVIDIA DMO Wrapper Filter is used to introduce NVIDIA Video and
 * Audio Encoder DMO in a DirectShow graph. This filter is included in
 * only for WM6 or when \c NV_INCLUDE_ENCODER_DMO environment
 * variable is set. This filter is meant to act as reference code
 * for custom DMO Wrapper Filters. Typically, Microsoft DMO Wrapper
 * Filter is preferred.
 *
 * @par GUID:
 * {98CA192F-771E-4de4-9F94-C4CAADF9721B}
 */
DEFINE_GUID(CLSID_NvDmoWrapper,
0x98ca192f, 0x771e, 0x4de4, 0x9f, 0x94, 0xc4, 0xca, 0xad, 0xf9, 0x72, 0x1b);

/*@}*/

/** @name Media Type GUIDs (AM_MEDIA_TYPE)
 * These are the media type (major type), media sub type (minor type),
 * and format type GUIDs used by capture and encode filters.
 */
/*@{*/

/**
 * @par Format type:
 * FORMAT_WaveFormatEx, Format structure: WAVEFORMATEX
 *
 * @par GUID:
 * {06153808-470D-4408-8815-39A5373BBCA8}
 */
/// {6972E3BB-3D6E-4c5d-AF6C-78B15E5CF70F}
DEFINE_GUID(MEDIASUBTYPE_QCELPAudio,
0x6972e3bb, 0x3d6e, 0x4c5d, 0xaf, 0x6c, 0x78, 0xb1, 0x5e, 0x5c, 0xf7, 0xf);

/// {F822742D-A1AB-4481-A6FD-3E2F256DC558}
DEFINE_GUID(MEDIASUBTYPE_EAACPLUSAudio,
0xf822742d, 0xa1ab, 0x4481, 0xa6, 0xfd, 0x3e, 0x2f, 0x25, 0x6d, 0xc5, 0x58);

/**
 * @par Format type:
 * FORMAT_VideoInfo, Format structure: VIDEOINFOHEADER
 * 
 * @par GUID:
 * {2CFC3DF4-CD5A-4a8f-BFAE-91D6B086EB90}
 */

/// {28e3a27d-5f48-4700-913b-4986608c8a8b}
DEFINE_GUID(MEDIATYPE_NvVideoBuffered,
0x28e3a27d, 0x5f48, 0x4700, 0x91, 0x3b, 0x49, 0x86, 0x60, 0x8c, 0x8a, 0x8b);

/// {451efe59-3bbd-43c7-9e67-511a835fa7f1}
DEFINE_GUID(MEDIATYPE_NvAudioBuffered,
0x451efe59, 0x3bbd, 0x43c7, 0x9e, 0x67, 0x51, 0x1a, 0x83, 0x5f, 0xa7, 0xf1);

/// {B0813207-7E3A-47d5-9E7E-BFC2E98EBB15}
DEFINE_GUID(MEDIATYPE_NvSourceAudio,
0xb0813207, 0x7e3a, 0x47d5, 0x9e, 0x7e, 0xbf, 0xc2, 0xe9, 0x8e, 0xbb, 0x15);

/// {2D316723-50DB-437D-865E-C2FD3B5B018D}
DEFINE_GUID(MEDIASUBTYPE_NvMMBuffer,
0x2d316723, 0x50db, 0x437d, 0x86, 0x5e, 0xc2, 0xfd, 0x3b, 0x5b, 0x1, 0x8d);

/**
 * This media subtype (MEDIASUBTYPE_Bayer) is used to collect raw bayer data
 * into a file as output. This would connect the mux file writer filter
 * directly to the still pin of NVIDIA Video Capture Filter.
 * This media type can be used for analyzing raw data from a camera.
 *
 * @par GUID:
 * {ED58F0CB-9F4D-4ace-B0A5-A9E84885B0A7}
 */
DEFINE_GUID(MEDIASUBTYPE_Bayer,
0xed58f0cb, 0x9f4d, 0x4ace, 0xb0, 0xa5, 0xa9, 0xe8, 0x48, 0x85, 0xb0, 0xa7);

/*@}*/

/** @name Interface GUIDs
 * These are the interface GUIDs (IIDs) used by capture and encode
 * filters.
 */
/*@{*/

/// {16841F1E-7B43-477f-8F37-D171AA98404B}
DEFINE_GUID(IID_INvDmoWrapper,
0x16841f1e, 0x7b43, 0x477f, 0x8f, 0x37, 0xd1, 0x71, 0xaa, 0x98, 0x40, 0x4b);

/*@}*/

/** @name Property Set GUIDs
 * These are the property set GUIDs used by capture and encode filters
 * to set properties.
 */
/*@{*/

/**
 * Property set GUID to set pin level properties. This is 
 * used by NVIDIA Video Capture Filter and NVIDIA Audio Capture Filter.
 *
 * @par GUID:
 * {E70028D6-3D62-475a-909F-B6883CF99141}
 *
 * The @b IKsPropertySet interface is used to set properties on
 * NVIDIA Video Capture Filter and NVIDIA Audio Capture Filter.
 * This interface is exposed on all pins of the two capture
 * filters. For more information, see MSDN at:
 *    <a href="http://msdn.microsoft.com/en-us/library/aa929907.aspx" 
 *       target="_blank">
 *       http://msdn.microsoft.com/en-us/library/aa929907.aspx</a>
 *
 * See ::NvPropertyId enumeration for all supported Property IDs.
 *
 * @par Example
 * The following example shows setting the pin level property on
 * NVIDIA Video Capture Filter. A similar method is used to set pin level
 * properties on NVIDIA Audio Capture Filter.
 *
 * @code
 *
 * // QueryInterface for IKsPropertySet from IPin interface or any interface
 * // exposed on the pin of NVIDIA Video Capture Filter. In this example, we
 * // will query IKsPropertySet from IPin interface of the capture
 * // pin present in NVIDIA Video Capture Filter.
 * pCapPin->QueryInterface(IID_IKsProperty, (void **)pCapPinPropSet);
 *
 * // Call IKsPropertySet::Set to set a pin-level property. In this example, we
 * // set rotation on the capture pin.
 * NvDdk2dTransform Transform = NvDdk2dTransform_Rotate180;
 * pCapPinPropSet->Set(
 *     PROPSET_NvPin,                   // Pin level property.
 *     NvPropertyId_2dTransform,        // Property identifier.
 *     NULL,
 *     0,
 *     &Transform,                      // Property value.
 *     sizeof(NvDdk2dTransform));       // Type of property value.
 *
 * @endcode
 */
DEFINE_GUID(PROPSET_NvPin,
0xe70028d6, 0x3d62, 0x475a, 0x90, 0x9f, 0xb6, 0x88, 0x3c, 0xf9, 0x91, 0x41);

/**
 * Property Set GUID to set filter-level properties. This is used by
 * NVIDIA Video Capture Filter and NVIDIA Audio Capture Filter.
 *
 * @par GUID:
 * {A211D09C-73BC-4fc8-B613-03471A9C69BF}
 *
 * The @b IKsPropertySet interface is used to set properties on
 * NVIDIA Video Capture Filter and NVIDIA Audio Capture Filter.
 * This interface is exposed on on the two capture filters.
 * For more information, see MSDN at:
 *    <a href="http://msdn.microsoft.com/en-us/library/aa929907.aspx" 
 *       target="_blank">
 *       http://msdn.microsoft.com/en-us/library/aa929907.aspx</a>
 *
 * All properties can be set anytime after the video or audio capture filter
 * is created. Most properties can be changed at run-time.
 *
 * See ::NvPropertyId enumeration for all supported Property IDs.
 *
 * @par Example
 * The following example shows setting the filter-level property on
 * NVIDIA Video Capture Filter. A similar method is used to set filter-level
 * properties on NVIDIA Audio Capture Filter.
 *
 * @code
 *
 * // QueryInterface for IKsPropertySet from IBaseFilter interface or any
 * // interface exposed on NVIDIA Video Capture Filter. In this example, we
 * // will query IKsPropertySet from IBaseFilter interface of NVIDIA
 * // Video Capture Filter.
 * pCapBaseFilter->QueryInterface(IID_IKsProperty, (void **)pCapFilterPropSet);
 *
 * // Call IKsPropertySet::Set to set a filter-level property. In this example,
 * // we set zoom.
 * NvU16 Zoom = 200;
 * pCapPinPropSet->Set(
 *     PROPSET_NvFilter,                // Filter-level property.
 *     NvPropertyId_Zoom,               // Property identifier.
 *     NULL,
 *     0,
 *     &Zoom,                           // Property value.
 *     sizeof(NvU16));                  // Type of property value.
 *
 * @endcode
 */
DEFINE_GUID(PROPSET_NvFilter,
0xa211d09c, 0x73bc, 0x4fc8, 0xb6, 0x13, 0x3, 0x47, 0x1a, 0x9c, 0x69, 0xbf);

/*@}*/

/** @name Custom Events
 * These are custom graph events sent by capture and encode related filters
 * to the application. Filters post these events to the filter
 * graph using the @b IMediaEventSink interface. Applications can
 * trap these events using the @b IMediaEvent and @b IMediaEventEx
 * interfaces.
 *              
 * For more information, see MSDN documentation at:
 * - <a href="http://msdn.microsoft.com/en-us/library/ms783650(VS.85).aspx"
 *       target="_blank">http://msdn.microsoft.com/en-us/library/ms783650(VS.85).aspx</a> for DirectShow event notification
 * - <a href="http://msdn.microsoft.com/en-us/library/aa915935.aspx"
 *       target="_blank">http://msdn.microsoft.com/en-us/library/aa915935.aspx</a> for @b IMediaEvent
 * - <a href="http://msdn.microsoft.com/en-us/library/aa928602.aspx"
 *       target="_blank">http://msdn.microsoft.com/en-us/library/aa928602.aspx</a> for @b IMediaEventEx
 */
/*@{*/

/**
 * This is the same as \c EC_USER, which is defined in evcode.h
 * to be 0x80000. All custom event codes should begin from \c EC_USER. 
 */
enum { EC_NV_USER = 0x8000 };

/**
 * This is the base for all graph events sent by capture and encode
 * related filters. This is to avoid conflicts with graph events sent by
 * playback and decode related filters.
 */
enum { EC_NV_ENC = 0x1000 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was able to achieve auto focus in response to half press.
 *
 * @par To receive this event:
 * -# Enable auto focus by setting \c AutoFocus to NV_TRUE using
 *    ::NvPropertyId_CameraProperties filter-level property.
 * -# Optionally set \c ExposureTime and \c WhiteBalance.
 * -# Enable half press by setting ::NvPropertyId_HalfPress
 *    filter-level property.
 */
enum { EC_NV_AUTO_FOCUS_ACHIEVED = EC_NV_USER + EC_NV_ENC + 0x1 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was able to achieve auto exposure in response to half press.
 *
 * @par To receive this event:
 * -# Enable auto exposure by setting \c ExposureTime to \a Auto using
 *    ::NvPropertyId_CameraProperties filter-level property.
 * -# Optionally set \c AutoFocus and \c WhiteBalance.
 * -# Enable half press by setting ::NvPropertyId_HalfPress
 *    filter-level property.
 */
enum { EC_NV_AUTO_EXPOSURE_ACHIEVED = EC_NV_USER + EC_NV_ENC + 0x2 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was able to achieve auto white balance in response to
 * half press.
 * 
 * @par To receive this event:
 * -# Enable auto white balance by setting \c WhiteBalance to \a Auto
 *    using ::NvPropertyId_CameraProperties filter-level property.
 * -# Optionally set \c AutoFocus and \c ExposureTime.
 * -# Enable half press by setting ::NvPropertyId_HalfPress
 *    filter-level property.
 */
enum { EC_NV_AUTO_WHITE_BALANCE_ACHIEVED = EC_NV_USER + EC_NV_ENC + 0x3 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was unable to achieve auto focus in response to half
 * press in a certain amount of time.
 * This event is sent instead of ::EC_NV_AUTO_FOCUS_ACHIEVED if
 * auto focus times out.
 */
enum { EC_NV_AUTO_FOCUS_TIMED_OUT = EC_NV_USER + EC_NV_ENC + 0x4 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was unable to achieve auto exposure in response to half
 * press in a certain aomount of time.
 * This event is sent instead of ::EC_NV_AUTO_EXPOSURE_ACHIEVED if
 * auto exposure times out.
 */
enum { EC_NV_AUTO_EXPOSURE_TIMED_OUT = EC_NV_USER + EC_NV_ENC + 0x5 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was unable to achieve auto focus in response to half
 * press in a certain amount of time.
 * This event is sent instead of ::EC_NV_AUTO_FOCUS_ACHIEVED if
 * auto focus times out.
 */
enum { EC_NV_AUTO_WHITE_BALANCE_TIMED_OUT = EC_NV_USER + EC_NV_ENC + 0x6 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the pause
 * still capture feature is enabled.
 * 
 * To enable the pause still capture feature, use the
 * ::NvPropertyId_PausePreviewAfterStillCapture property. After
 * a still capture is completed, NVIDIA Video Capture Filter
 * sends this event. The application can trap this event and
 * then pause the filter graph. During this time, the application
 * can show the captured image to the user for confirmation.
 * After confirmation, the application can then run the filter
 * graph again.
 * 
 */
enum { EC_NV_AUTO_PAUSE_STILL_CAPTURE_DONE = EC_NV_USER + EC_NV_ENC + 0x7 };

/**
 * This event is sent by NVIDIA Mux File Writer when it is done capturing
 * a still frame.
 *
 * An application can trigger NVIDIA's Video Capture Filter's still pin using
 * the <b>IAMVideoControl::SetMode</b> method. When triggered, the capture filter
 * delivers a still frame to the downstream filter. When NVIDIA Mux File Writer
 * filter is done with the still frame, it sends this event. The event is sent
 * once per trigger.
 *
 * For more information on <b>IAMVideoControl::SetMode</b>, see MSDN documentation at:
 * <a href="http://msdn.microsoft.com/en-us/library/aa915574.aspx"
 * target="_blank">http://msdn.microsoft.com/en-us/library/aa915574.aspx</a>
 */
enum { EC_NV_CAP_FILE_COMPLETED = EC_NV_USER + EC_NV_ENC + 0x8 };

/**
 * This event is sent by NVIDIA Mux File Writer for invalid user information 
 * or when User Data Size Exceeds 216 bytes. This is a warning message.
 * Mux Writer still continue to write data.
 */
enum { EC_NV_MUX_WRITER_INVALID_USER_DATA = EC_NV_USER + EC_NV_ENC + 0x9 };

/**
 * This event is sent by NVIDIA Mux File Writer when it has exhausted the 
 * 3GP File Size of 2 GB (Or File Size limit specified by user) as dictated 
 * by ISO MP4 version 0 which NVIDIA Mux File Writer supports. Upon receival
 * of this event Test can reinitialize the file name to continue dumping ]
 * the AV-Recorded data by setting property through ::NvPropertyId_MuxWriterReinitializeFilename.
 */
enum { EC_NV_MUX_WRITER_FILESIZE_LIMIT_EXCEED = EC_NV_USER + EC_NV_ENC + 0xA };

/**
 * This event is sent by NVIDIA Encoder Filter when it gets Unsupported resolutions 
 * property through ::EC_NV_RESOLUTION_NOT_SUPPORTED.
 */
enum { EC_NV_RESOLUTION_NOT_SUPPORTED = EC_NV_USER + EC_NV_ENC + 0xB };

/**
 * This event is sent by NVIDIA Video Capture Filter.
 * This will be used for non-fatal errors such as bad calibration data
 * or insufficient memory to use advanced features.
 * through ::EC_NV_VIDEO_CAPTURE_WARNING.
 * The type of warning will be send in param1 of Event Notify call.
 * Please refer NvMMBlockWarning in nvmm_event.h for all possible warning
 * associted with this notification.
 */
enum { EC_NV_VIDEO_CAPTURE_WARNING = EC_NV_USER + EC_NV_ENC + 0xC };

/**
 * This event is sent by NVIDIA Writer Filter.
 * This will be used to notify that BaseNumber of encoded file
 * has exceeded the value allowed by DCF i.e 9999
 * through ::EC_NV_MUX_WRITER_DCF_BASE_NUMBER_EXCEEDED.
 */
enum { EC_NV_MUX_WRITER_DCF_BASE_NUMBER_EXCEEDED = EC_NV_USER + EC_NV_ENC + 0xD };
/**
 * This event is sent by NVIDIA Video Capture Filter.
 * This will be used for SetaAttribute errors 
 * through ::EC_NV_SETATTRIBUTE_ERROR.
 */
enum { EC_NV_SETATTRIBUTE_ERROR = EC_NV_USER + EC_NV_ENC + 0xE };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter is ready for another still trigger request.
 */
enum { EC_NV_STILL_CAPTURE_READY = EC_NV_USER + EC_NV_ENC + 0xF };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter is ready for another video capture request.
 */
enum { EC_NV_VIDEO_CAPTURE_READY = EC_NV_USER + EC_NV_ENC + 0x10 };

#ifndef EC_CAP_FILE_WRITE_ERROR
#define EC_CAP_FILE_WRITE_ERROR                 0x104
#endif

/*@}*/

/**
 * Defines property IDs used to identify
 * properties of NVIDIA Video Capture Filter and NVIDIA Audio
 * Capture Filter. There are 2 types of Property ID sets:
 * - Filter-level property set identified by ::PROPSET_NvFilter, and
 * - Pin level property set identified by ::PROPSET_NvPin.
 *              
 * These properties are set using the IKsPropertySet interface
 * exposed by NVIDIA Video and NVIDIA Audio Capture Filter at pin and
 * filter level.
 */
typedef enum
{

    /**
     * Filter-level property to set camera properties.
     * Property value type is ::NvCameraProperties. Sets various camera
     * properties like auto focus, exposure time, white balance, etc.
     */
    NvPropertyId_CameraProperties = 4,

    /**
     * Filter-level property to set color properties.
     * Property value type is ::NvColorProperties. Sets various color
     * properties like hue, saturation, brightness, etc. 
     */
    NvPropertyId_ColorProperties,

    /**
     * Filter-level property to set zoom.
     * Property value type is NvU16. NVIDIA Video Capture Filter
     * supports zoom values from 100 to 800 (1.0-8.0) up to 8x digital
     * zoom.
     */
    NvPropertyId_Zoom,

    /**
     * Filter-level property to set smooth zoom time, in milliseconds.
     * Property value type is NvU32.
     */
    NvPropertyId_SmoothZoomTime,

    /**
     * Filter-level property to set smooth zoom abort. (Get is not
     * supported on this filter-level property.)
     * Property value type is NvBool.
     */
    NvPropertyId_SmoothZoomAbort,

    /**
     * Reserved.
     */
    NvPropertyId_ZoomMult,

    /**
     * Pin and filter-level property to set 2D transform.
     * Property value type is \c NvDdk2dTransform. This property is
     * supported at pin level as well as filter level. When set at the
     * filter level, NVIDIA Video Capture Filter will set the same
     * transform value on all of its pins.
     * @note Filter level does not support Get on this property. Pin
     * level supports both Get and Set.
     *
     * Supported transform values are:
     * - NvDdk2dTransform_None
     * - NvDdk2dTransform_Rotate90
     * - NvDdk2dTransform_Rotate180
     * - NvDdk2dTransform_Rotate270
     * - NvDdk2dTransform_FlipHorizontal (frame-flipping)
     * - NvDdk2dTransform_FlipVertical (frame-mirroring)
     * - NvDdk2dTransform_Transpose
     * - NvDdk2dTransform_InvTranspose
     *
     * The above values are defined in nvddk_2d.h
     */
    NvPropertyId_2dTransform,

    /**
     * Read-only filter-level property to get the current frame rate.
     * Property value type is NvF32. Used to get frame rate at which
     * NVIDIA Video Capture Filter is currently delivering frames.
     *
     * NVIDIA Video Capture Filter delivers frames between values
     * specified by ::NvPropertyId_FrameRateRange
     */
    NvPropertyId_CurrentFrameRate = 13,

    /**
     * Filter-level property to enable or disable image stabilization
     * Property value type is NvBool. Set to NV_TRUE to enable or
     * NV_FALSE to disable image stabilization.
     *
     * @note Turning this on implicitly disables video stabilization.
     */
    NvPropertyId_ImageStabilization,

    /**
     * Filter-level property to enable or disable video stabilization
     * Property value type is NvBool. Set to NV_TRUE to enable or
     * NV_FALSE to disable video stabilization.
     *
     * @note Turning this on implicitly disables image stabilization.
     */
    NvPropertyId_VideoStabilization,

    /**
     * Filter-level property to enable or disable image chroma filtering
     * Property value type is NvBool. Set to NV_TRUE to enable or
     * NV_FALSE to disable image chroma filtering.
     */
    NvPropertyId_ImageChromaFiltering,

    /**
     * Filter-level property to select active imager.
     * Property value type is NvU64. If the device has more
     * than 1 imagers, then this property is used to select between
     * them. The property values are numbers that identify the imager
     * on the device.
     *
     * @pre This property should be set before running the graph and should
     * be the first property to be set. If this property is set after setting
     * any other property that results in changing the currently selected
     * sensor, then the camera is reinitialized with the new sensor.
     * Reinitialization resets all properties to defaults. Hence, this property
     * must be set first.
     */
    NvPropertyId_ActiveImager,

    /**
     * Filter-level property to set and get the auto focus region.
     * The property value type is ::NvAutoFocusRegionNormalized. This property
     * can also be set at pin-level. When used at the pin level, the property
     * value type is ::NvAutoFocusRegion. Setting this property for one pin
     * will proportionately change auto focus region of all other pins since
     * there can be only auto focus region applied to all pins.
     *
     * Auto focus region will work only when auto focus is enabled. To enable
     * auto focus, set NvCameraProperties::AutoFocus to NV_TRUE and
     * set it using the ::NvPropertyId_CameraProperties
     * property ID.
     *
     * When specified at the filter level, the rectangle coordinates are
     * normalized to -1.0 to 1.0 on the X and Y axis. When specified at the
     * pin level, the rectangle coordinates are those of the actual frame.
     * As an example, if the video is 320 x 240 pixels and the application
     * wants to specify an auto focus region of 0, 0, 160, 120 (left half of
     * the video) then at filter level, the coordinates will be
     * -1.0, -1.0, 0.0, 0.0. On the other hand when specified at the pin level
     * then the coordinates will be 0, 0, 160, 120. This is so because at
     * pin level, the resolutions of preview, capture, and still can be
     * different. The pin level property is also specified for convenience.
     */
    NvPropertyId_AutoFocusRegion,

    /**
     * Filter-level and pin-level property to set cropping rectangle
     * Property value type is ::NvRect. Sets the cropping rectangle.
     * NVIDIA Video Capture Filter will crop all frames to this
     * rectangle before delivering them. When set at filter level, the
     * specified crop rectangle is applied to all 3 pins of the filter.
     * @note Filter level does not support Get on this property. Pin
     * level supports both Get and Set.
     */
    NvPropertyId_CropRect,

    /**
     * Filter-level property to set application type.
     * Property value type is ::NvAppType enumeration. NVIDIA Video
     * Capture filter manipulates time-stamps based on the application
     * type: Camcorder or Video Telephony (VT).
     *
     * @pre This property should be set before connection.
     */
    NvPropertyId_AppType,

    /**
     * Filter-level property to set edge enhancement (sharpness).
     * Property value type is ::NvEdgeEnhanceSettings structure.
     */
    NvPropertyId_EdgeEnhancement,

    /**
     * Filter-level property to repeat frames.
     * Property value type is ::NvRepeatedFrameSettings structure. When
     * enabled, NVIDIA Video Capture Filter will continuously deliver
     * the same frame until disabled. This property is also used to
     * continuously deliver an RGB565 or YUV420 (YV12) image provided
     * by the application.
     */
    NvPropertyId_RepeatFrame,

    /**
     * Filter-level property to enable and disable half press. (Get is not
     * supported on this filter-level property.)
     * Property value type is NvBool.
     * 
     * When set to NV_TRUE, NVIDIA Video Capture Filter will start
     * converging (achieving) auto focus, auto exposure, and/or auto
     * white balance properties. If the capture filter achieves one or
     * more of these properties then it will lock settings of those
     * properties. If the capture filter was unable to achieve one or
     * more of those properties in a certain time then it times out
     * for that property.
     * 
     * When set to NV_FALSE, NVIDIA Video Capture Filter will
     * unlock settings for auto focus, auto exposure, and/or auto
     * white balance properties.
     * 
     * NVIDIA Video Capture Filter notifies the application about
     * those properties that were achieved and those that timed out
     * using graph events. The following lists shows the different
     * events that are sent.
     * - Auto focus achieved (::EC_NV_AUTO_FOCUS_ACHIEVED)
     * - Auto focus timed out (::EC_NV_AUTO_FOCUS_TIMED_OUT)
     * - Auto exposure achieved (::EC_NV_AUTO_EXPOSURE_ACHIEVED)
     * - Auto exposure timed out (::EC_NV_AUTO_EXPOSURE_TIMED_OUT)
     * - Auto white balance achieved (
     *      ::EC_NV_AUTO_WHITE_BALANCE_ACHIEVED)
     * - Auto white balance timed out (
     *      ::EC_NV_AUTO_WHITE_BALANCE_TIMED_OUT)
     * 
     * See documentation for these events for information on how the
     * events can be trapped by applications.
     * 
     * Although half press is used with auto focus, auto exposure, and
     * auto white balance properties, an application can choose to
     * enable any one or any combination of these properties. When
     * any of these properties is disabled then half press will not
     * attempt to achieve that property.
     * 
     * Setting auto focus, auto exposure, and auto white balance:
     * 
     * - To enable auto focus, set NvCameraProperties::AutoFocus 
     *      to NV_TRUE, and set it using
     *      ::NvPropertyId_CameraProperties property ID.
     * - To enable auto exposure, set NvCameraProperties::ExposureTime 
     *      to ::NV_EXPOSURETIME_AUTO, and set it using
     *      \c NvPropertyId_CameraProperties property ID.
     * - To enable auto white balance, set NvCameraProperties::WhiteBalance 
     *      to ::NvWhiteBalance_Auto, and set it using
     *      \c NvPropertyId_CameraProperties property ID.
     * 
     * @par See also
     * Custom Events
     */
    NvPropertyId_HalfPress,

    /**
     * Filter-level property to set timeout value for half press.
     * Property value type is NvU32.
     * 
     * Requires a MAX time-out value in miliseconds, for which the camera can
     * converge (achieve) auto focus, auto exposure, and/or auto white balance
     * property before timing out.
     * 
     * This is valid only when ::NvPropertyId_HalfPress is NV_TRUE, and should
     * be set before \c NvPropertyId_HalfPress.
     */
    NvPropertyId_HalfPressTimeOut,

    /**
     * Filter-level property to enable Exif information.
     * Property value type is NvBool. Set to NV_TRUE for NVIDIA Video
     * Capture Filter to generate Exif information. This information
     * is used by NVIDIA Video Encoder Filter to embed Exif
     * information in JPEG images.
     */
    NvPropertyId_EnableExifInfo,

    /**
     * Filter-level property to enable pre capture converge.
     * Property value type is NvBool. This property functions
     * similar to half press. When set to NV_TRUE, NVIDIA Video
     * Capture Filter will achieve auto focus, auto exposure, or
     * auto white balance even before delivering the first frame.
     *
     * @pre This property should be set before connection.
     *
     * @sa     NvPropertyId_HalfPress
     */
    NvPropertyId_EnablePreCaptureConverge,

    /**
     * Filter-level property to change recording volume.
     * Property value type is NvU32 and is for NVIDIA Audio Capture
     * Filter. Property values range between 0 to UINT max.
     */
    NvPropertyId_RecordVolType,

    /**
     * Filter-level property to set automatic gain control (AGC).
     * Property value type is ::NvRecordAgcProperties. This property is
     * for NVIDIA Audio Capture Filter.
     */
    NvPropertyId_RecordAGC,

    /**
     * Filter-level property to set parametric equalizer filter.
     * Property value type is ::NvParametricEqFilterProperties. This
     * property is for NVIDIA Audio Capture Filter.
     */
    NvPropertyId_ParametricEqFilter,

    /**
     * Filter-level property to set pause after still capture feature.
     * Property value type is NvBool. When set to NV_TRUE, NVIDIA
     * Video Capture Filter sends an
     * ::EC_NV_AUTO_PAUSE_STILL_CAPTURE_DONE event to the application
     * whenever a still capture is triggered. This event can be
     * used by the application to pause the filter graph, ask the user
     * for save/delete confirmation, and run the filter graph again.
     */
    NvPropertyId_PausePreviewAfterStillCapture,

    /**
     * Filter-level property to enable preview stream.
     * Property value type is NvBool.
     */
    NvPropertyId_CameraAttributePreview,

    /**
     * Filter-level property to set white balance correlated color
     * temperature (CCT) range. Property value type is ::NvCaptureRange.
     * Auto white balance will normally search a large set of illuminants to
     * estimate the CCT of a scene. This range can be limited to a subset
     * of the defined illuminants by setting this parameter. The
     * \c NvCaptureRange structure's low and high values in the range
     * specify the minimum and maximum CCT values to be searched. Legal values
     * are integers in units of degrees K. To select a specific illuminant,
     * the low and high members should be set to identical values. By default,
     * the low range value is 0, and the high value is MAX_INT.
     */
    NvPropertyId_WhiteBalanceCCTRange,

    /**
     * Filter-level property to set file base number. This is required
     * for burst mode image capture. The base number will be incremented
     * and appended to the base file name for each burst mode image
     * captured. This is set on NVIDIA Mux File Writer.
     * Property value type is NvU32.
     * Property values range between 0 to UINT max.
     */
    NvPropertyId_FileBaseNumber,

    /**
     * Filter-level property to set file naming mode.
     * We use DCM File naming standard. To disable this 
     * mode, set this property on NVIDIA Mux File Writer.
     * Property value type is NvBool.
     * Property values is NV_FALSE to disable DCM file naming.
     */
    NvPropertyId_DCFFileMode,

    /**
     * Mux filter-level property to set the upper limit for encoded file size.
     * Property value type is NvU64. Size must be specified in bytes.
     * Default upper limit of encoded file size is 2 GB.
     */
    NvPropertyId_EncodedFileSize,

    /**
     * Filter-level property to set number of JPEG images to be captured in
     * burst mode. Set this property on NVIDIA Video Capture Filter.
     * Property value type is NvU32.
     * To cancel an on going burst capture, set count to 0 on this property and
     * trigger still pin.
     */
    NvPropertyId_JpegBurstModeImageCount,
    
    /**
     * Capture filter property to describe camera metering mode.
     * Metering modes are valid only when auto exposure (AE) is specified,
     * i.e., not valid for manual exposure. When AE is not specified,
     * spot metering mode is the default. Metering mode values
     * are defined in the ::NvCameraMeteringMode enumeration.
     */
    NvPropertyId_CameraMeteringMode,

    /** 
     * Filter-level property to get the number of audio frames written.
     * This is a read-only property supported by NVIDIA Mux File Writer. 
     * Property value type is NvU64. 
     */
    NvPropertyId_AudioEncodedFramesWritten,

    /** 
     * Filter-level property to get the number of video frames written .
     * This is a read-only property supported by NVIDIA Mux File Writer. 
     * Property value type is NvU64. 
     */
    NvPropertyId_VideoEncodedFramesWritten,
    
    /**
     * Filter-level property to get/set the camera profile. Applications can 
     * enable profile before Dshow graph runs and get profile data at runtime, 
     * but this property does not support setting it after capture is runing.
     * Property value type is ::NvCaptureProfileRec.
     */
    NvPropertyId_CameraProfile,

    /**
     * NVIDIA Mux File Writer property to reinitialize filename upon receipt
     * of ::EC_NV_MUX_WRITER_MORE_DATA_REJECT or 
     * ::EC_NV_MUX_WRITER_FILESIZE_LIMIT_EXCEED events.
     *
     * This property can also be used by the application to have a new file 
     * created after every camera stream pause, followed by setting the property 
     * to NVIDIA Video Encoder to \c ENCODER_VIDEO_IFRAME_INSERT. So in this case,
     * the programming sequence would look like the following:
     * After resumption from \c ControlStream(Stop), i.e. on 
     * \c ControlStream(Start):
     * -# setProperty ::NvPropertyId_MuxWriterReinitializeFilename
     * -# setProperty ::ENCODER_VIDEO_IFRAME_INSERT to encoder 
     *
     * This is a write-only property supported by NVIDIA Mux File Writer.
     */
    NvPropertyId_MuxWriterReinitializeFilename,

    /**
     * Filter-level property to get/set frame rate range of NVIDIA Video
     * capture filter. Property value type is ::NvCaptureRange *. Sets the
     * minimum and maximum frame rate at which the capture filter delivers
     * frames.
     *
     * NVIDIA Video Capture filter supports auto frame rate, where it
     * dynamically changes frame rate between a minimum value specified by
     * \c NvCaptureRange::Low and a maximum value specified by
     * \c NvCaptureRange::High. To make the capture filter deliver frames at
     * a constant frame rate, set \c Low and \c High members of \c NvCaptureRange
     * structure to the same value.
     *
     * Maximum possible frame rate is dependent on the camera sensor. Values
     * above the maximum frame rate supported by the camera sensor are clipped
     * to the maximum value.
     */
    NvPropertyId_FrameRateRange,

     /**
     * Filter-level property to get a block of custom info from the imager.
     * Property value type is ::NvCaptureCustomInfo. This is a read-only property.
     * The block of info that is returned is private and is known to the NvODM
     * implementation.
     */
    NvPropertyId_CustomizedBlockInfo,

    /**
     * Filter-level property to get/set exposure time range of NVIDIA Video
     * capture filter. Property value type is NvCaptureRange *. Sets the
     * minimum and maximum exposure time or iso sensitivty at which the
     * capture filter's auto exposure picks values from.
     *
     * NVIDIA Video Capture filter supports auto exposure, where it
     * dynamically changes exposure time and sensor's sensitivty between a
     * minimum value specified by NvCaptureRange::Low and a
     * maximum value specified by \c NvCaptureRange::High. 
     *
     * Minimum and maximum possible exposure times and iso gains are dependent
     * on the camera sensor. Values above the maximum and below the minimum 
     * supported by the camera sensor are clipped.
     * 
     * For ExposureTimeRange:
     * Values are in units of milliseconds. To limit the exposure to nothing
     * slower than 1/30 second, a setting like {0, 33} would be used. Zero would
     * be replaced with the sensors fastest exposure limit by the driver.
     * For ISORange:
     * Values are in units of ISO. To limit the ISO gain to nothing more than
     * ISO400, a setting like {0, 400} would be used. Zero would be replaced
     * with the sensor's lowest supported gain limit by the driver.
     * Since the Auto exposure algorithm is 'exposure time priority', limiting
     * the ISO range will only dim the image if the ISO is limited by setting
     * the range. The exposure time is not recalculated based off of the
     * ISO range limit.
     */
    NvPropertyId_ExposureTimeRange,
    NvPropertyId_ISORange,

    /**
     * Filter-level property to get/set how much flash white balance
     * correction to use when capturing flashed images. The property type
     * is NvF32, and the range is 0 to 1. For example, at .30,
     * it would use 30% of the flash color correction and 70% of the
     * previously-calculated ambient color correction.
     *
     * @note If the ambient scene was very dark, AWB may choose to
     * mix in more flash correction than specified. Setting this property
     * to zero disables this behavior.
     */
    NvPropertyId_FlashMixingRatio,

    /**
     * NVIDIA Mux File Writer property to program 32 bit Atom compatible 
     * 3GP Files. Default is 64 bit Atom. To Enable 32 bit atom Mode
     * This property needs to be set to TRUE, i.e boolean value one.
     * This is a write-only property supported by NVIDIA Mux File Writer.
     */
    NvPropertyId_MuxWriterEnable64To32BitAtomMode,

    /**
     * Filter-level property to set warmth.
     * Property value type is NvF32. NVIDIA Video Capture Filter
     * supports warmth values from -10 to 10
     */
    NvPropertyId_Warmth,

     /**
     * NVIDIA Mux File Writer property to set user data to go in Udta 
     * Atom of 3GP/MP4 sink files.
     * Application can define any of the userconfig type as stated in enum
     * Nv3GPWriterUserDataType to go in Udta Atom of 3GP/MP4.
     * Apllication will setproperty to MUX_WRITER with BYREF Variant , passing 
     * on pointer of completely/properly filled in structure
     * NvWriterUserDataConfig. 
     * It is sole responsiblity of the application to supply right
     * data in NvWriterUserDataConfig::pUserData as it will simply be sticthed 
     * in the sink 3GP/Mp4 file by NVIDIA FileWriter without 
     * checking/processing its content, there upon if the resulting 3GP File is 
     * not playable that will imply something is wrong in application supplied
     * NvWriterUserDataConfig::pUserData.
     * 
     */
    NvPropertyId_MuxWriterUserData,

    /**
     * While the auto exposure algorithm is running it will internally track
     * the calculated brightness of the scene, which is the product of the
     * average scene luminance and the reciprocal of the exposure settings
     * used. If a scene is low key but in bright light, this value will be
     * smaller than a high key scene in bright light. The lowest value will
     * be low key scenes with very long exposures; the highest value will be
     * high key scenes in bright light. They property type is NvF32 and this
     * is read-only.
     */
     NvPropertyId_SceneBrightness,

    /**
     * Setting the thumbnail quality improvement. When it's set to off, there
     * won't be any improvement. When it's set to best/fast, thumbnail quality
     * improvement will be applied to the still capture and every frame of 
     * a burst capture. When it's set to LastFrameBest/LastFrameFastest,
     * thumbnail quality improvement will be applied to the still capture and
     * the last frame of a burst capture.
     * The property type is NvThumbnailQualityImprovement.
     */
    NvPropertyId_ThumbnailQualityImprovement,

    /**
     * Filter-level property to set the convergence speed of the AE and AWB
     * algorithms. It is a write-only property.
     *
     * Property type: NvU8*
     *
     * Only the specified two strings are allowed :
     * - "ae.ConvergeSpeed=%f"
     * - "awb.ConvergeSpeed=%f"
     *
     * where %f is a float value.
     *
     * The float value for 
     * - ae must be [0.01-1.0] 
     * - awb must be [0.01-1.0]
     *
     * If the strings passed do not match any of the above 2 strings then
     * an error is returned.
     */
    NvPropertyId_CameraConfiguration,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvPropertyId_Force32 = 0x7FFFFFFF
} NvPropertyId;

/**
 * Enumeration to set effects.
 * Set NvColorProperties::Effect and set using
 * the ::NvPropertyId_ColorProperties property ID.
 */
typedef enum
{
    /** Disables all effects. Default. */
    NvEffect_None = 1,
    /** Enables emboss effect. */
    NvEffect_Emboss,
    /** Enables negative effect. */
    NvEffect_Negative,
    /** Enables solarize effect. */
    NvEffect_Solarize,
    /** Enables posterize effect. */
    NvEffect_Posterize,
    /** Enables sepia effect. */
    NvEffect_Sepia,
    /** Enables gray-scale effect. */
    NvEffect_GrayScale,
    /**
     * Enables color matrix for custom color effects. Specify the color matrix
     * using the ColorMatrix member of the ::NvColorProperties structure.
     */
    NvEffect_ColorMatrix,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvEffect_Force32 = 0x7FFFFFFF
} NvEffect;

/**
 * Enumeration that lists supported white balance settings
 * Specify NvCameraProperties::WhiteBalance and set
 * using ::NvPropertyId_CameraProperties property ID.
 * @sa NvPropertyId_HalfPress
 */
typedef enum
{
    /** Switches off white balance. */
    NvWhiteBalance_Off = 1,
    /** Sets white balance to auto. */
    NvWhiteBalance_Auto,
    /** Sets white balance to tungsten. */
    NvWhiteBalance_Tungsten,
    /** Sets white balance to Fluorescent. */
    NvWhiteBalance_Fluorescent,
    /** Sets white balance to sunny. */
    NvWhiteBalance_Sunny,
    /** Sets white balance to shade. */
    NvWhiteBalance_Shade,
    /** Sets white balance to cloudy. */
    NvWhiteBalance_Cloudy,
    /** Sets white balance to incandescent. */
    NvWhiteBalance_Incandescent,
    /** Sets white balance to flash. */
    NvWhiteBalance_Flash,
    /** Sets white balance to horizon. */
    NvWhiteBalance_Horizon,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvWhiteBalance_Force32 = 0x7FFFFFFF
} NvWhiteBalance;

/**
 * Enumeration to ISO sensitivity.
 * Specify NvCameraProperties::Sensitivity and set
 * using ::NvPropertyId_CameraProperties property ID. ISO sensitivity is the
 * sensor's gain multiplied by its nominal ISO sensitivity.
 */
typedef enum
{
    /**
     * Sets ISO sensitivity to auto. when set to auto, querying this property
     * will return the ISO sensitivity value computed by the auto-exposure
     * algorithm.
     */
    NvISO_Auto =   -1,
    /** Sets gain x nominal ISO sensitivity to 50. */
    NvISO_50   =   50,
    /** Sets gain x nominal ISO sensitivity to 100. */
    NvISO_100  =  100,
    /** Sets gain x nominal ISO sensitivity to 200. */
    NvISO_200  =  200,
    /** Sets gain x nominal ISO sensitivity to 400. */
    NvISO_400  =  400,
    /** Sets gain x nominal ISO sensitivity to 800. */
    NvISO_800  =  800,
    /** Sets gain x nominal ISO sensitivity to 1600. */
    NvISO_1600 = 1600,
    /** Sets gain x nominal ISO sensitivity to 3200. */
    NvISO_3200 = 3200,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvISO_Force32 = 0x7FFFFFFF
} NvISOSensitivity;

/**
 * Enumeration to set exposure time automatically.
 * Specify NvCameraProperties::ExposureTime and set
 * using ::NvPropertyId_CameraProperties property ID. 
 * \c ExposureTime can be set to any whole number value
 * representing quantum of time in milliseconds; a value less than 1 msec 
 * programs the camera to evaluate exposure time automatically.
 * This enum has only one value defined to distinguish it for this
 * special case of Auto Exposure Time.
 * @sa NvPropertyId_HalfPress.
 */
enum { NV_EXPOSURETIME_AUTO = -1 };

/**
 * Enumeration to set flash mode.
 * Specify NvCameraProperties::FlashMode and set using
 * ::NvPropertyId_CameraProperties property ID.
 */
typedef enum
{
    /** Disables flash. */
    NvFlashMode_Off = 1,

    /** Flash is forced on. */
    NvFlashMode_Torch,

    /** Flash is turned on for video-capture. */
    NvFlashMode_Movie,

    /** Manual flash control for still image capture. */
    NvFlashMode_Still_Manual,
      
    /** Automatically enables flash when required for still image capture. */
    NvFlashMode_Still_Auto,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvFlashMode_Force32 = 0x7FFFFFFF
} NvFlashMode;

/**
 * Enumeration type for quality level of the embedded thumbnail.
 * Used with the ::ENCODER_THUMBNAIL_QUALITY video encoder property.
 */
typedef enum
{
    /** Sets JPEG thumbnail quality to 12 (lowest). */
    NvJPEGThumbnailQuality_12 = 1,
    /** Sets JPEG thumbnail quality to 25. */
    NvJPEGThumbnailQuality_25,
    /** Sets JPEG thumbnail quality to 50. */
    NvJPEGThumbnailQuality_50,
    /** Sets JPEG thumbnail quality to 75. */
    NvJPEGThumbnailQuality_75,
    /** Sets JPEG thumbnail quality to 100 (highest). */
    NvJPEGThumbnailQuality_100,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvJPEGThumbnailQuality_Force32 = 0x7FFFFFFF
} NvJPEGThumbnailQuality;

/**
 * Enumeration type for flicker control.
 * Specify NvCameraProperties::FlickerControl and set using
 * ::NvPropertyId_CameraProperties property ID.
 */
typedef enum
{
    /** Disables Flicker Control */
    NvFlickerControl_Off = 1,
    /** Sets Flicker Control to Auto */
    NvFlickerControl_Auto,
    /** Sets Flicker Control to 50 Hz */
    NvFlickerControl_50,
    /** Sets Flicker Control to 60 Hz */
    NvFlickerControl_60,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvFlickerControl_Force32 = 0x7FFFFFFF
} NvFlickerControl;


typedef enum
{
    /** No Thumbnail Improvement */
    NvThumbnailQualityImprovement_Off= 1,

    /** Best Thumbnail Improvement for All Still Captures */
    NvThumbnailQualityImprovement_Best,

    /** Fastest Thumbnail Improvement for All Still Captures */
    NvThumbnailQualityImprovement_Fastest,

    /** Best Thumbnail Improvement for the Last Still Capture */
    NvThumbnailQualityImprovement_LastFrameBest,

    /** Fatest Thumbnail Improvement for the Last Still Capture */
    NvThumbnailQualityImprovement_LastFrameFastest,

    NvThumbnailQualityImprovement_Force32 = 0x7FFFFFFF,

} NvThumbnailQualityImprovement;



/**
 * Sets frame rate as fast as possible.
 * When ::NvPropertyId_FrameRateRange High and Low values are set to
 * \c FRAME_RATE_MAX, then NVIDIA Video Capture Filter will deliver frames at
 * the maximum frame-rate.
 */
enum { FRAME_RATE_MAX = 0xFFFF };

/**
 * Defines parametric EQ filter types.
 */
typedef enum
{
    /** Sets filter type to band-pass IIR. */
    NvPEF_BANDPASS_IIR_FILTER = 1,
    /** Sets filter type of high pass IIR. */
    NvPEF_HIGHPASS_IIR_FILTER,
    /** Sets filter type of low pass IIR. */
    NvPEF_LOWPASS_IIR_FILTER,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvPEF_Filter_Force32 = 0x7FFFFFFF
} NvPEF_Filter;

/**
 * Maximum size of decode configuration info that can be returned by
 * NVIDIA Video Encoder Filter.
 */
enum { NV_VIDEOENC_DCI_SIZE = 80 };

/**
 * Structure for different supported camera properties.
 * Used with the ::NvPropertyId_CameraProperties property ID.
 */
typedef struct NvCameraPropertiesRec
{
    /** Set to NV_TRUE to enable anti-flicker, or NV_FALSE otherwise. */
    NvFlickerControl FlickerControl;
    /** Set to NV_TRUE to enable auto-whitebalance, or NV_FALSE otherwise. */
    NvBool AutoWhiteBalance;
    /** Set to auto or different manual settings. */
    NvWhiteBalance WhiteBalance;
    /** Set to NV_TRUE to enable auto-exposure, or NV_FALSE otherwise. */
    NvBool AutoExposure;
    /** Set to the available exposure time. The value is quantum of time 
      *  in milliseconds. */
    NvS32 ExposureTime;
    /** Set to NV_TRUE for to allow auto-exposure to select the values for
      * exposure time, ignoring the ExposureTime parameter. */
    NvBool ExposureTimeIsAuto;
    /** Set to auto or one of supported values. */
    NvISOSensitivity Sensitivity;
    /** Set to NV_TRUE for to allow auto-exposure to select the values for
      * iso sensitivity, ignoring the Sensitivity parameter. */
    NvBool SensitivityIsAuto;
    /** Set to NV_TRUE to specify that AutoFocus should be run for
      * subsequent calls to HalfPress, unless ManualFocus is also
      * set to NV_TRUE */
    NvBool AutoFocus;
    /** Set to NV_TRUE to specify that the FocusPosition should be used
      * as a manual focus position, and AutoFocus should not be run. */
    NvBool ManualFocus;
    /** Set to position number or 0 as default. */
    NvS32 FocusPosition;
    /** Set off/torch/movie/still_manual/still_auto to enable/automate/disable flash. */
    NvFlashMode FlashMode;
    /** Value in [0..1] range to control torch output level. */
    NvF32 ManualTorchLevel;    
    /** Value in [0..1] range to control flash output level. */
    NvF32 ManualFlashLevel;    
    /** Set to NV_TRUE to enable auto-focus, or NV_FALSE otherwise. */
} NvCameraProperties;

/**
 * Maximum number of matrix rows.
 */
enum { CAPTURE_MATRIX_ROWS = 3 };

/**
 * Maximum number of matrix columns.
 */
enum { CAPTURE_MATRIX_COLS = 3 };

/**
 * Structure for different supported color properties.
 * Used with the ::NvPropertyId_ColorProperties property ID.
 */
typedef struct NvColorPropertiesRec
{
    /** Effect mode. */
    NvEffect Effect;
    /** Hue between 0 to 359. */
    NvF32 Hue;
    /** Saturation between 0% to 200%. */
    NvF32 Saturation;
    /** Bightness between 0% to 300%. */
    NvF32 Brightness;
    /** Contrast between 0% to 100%. */
    NvU16 Contrast;
    /** ColorMatrix - valid only if Effect = NvEffect_ColorMatrix */
    NvF32 ColorMatrix[CAPTURE_MATRIX_ROWS][CAPTURE_MATRIX_COLS];
} NvColorProperties;

/**
 * Structure for different Exif items that can be set.
 * This structure is a sub-set of Exif information that must come
 * from the application. Set using the ::ENCODER_EXIF_PROPERTIES
 * property on NVIDIA Video Encoder Filter configured for
 * JPEG encoding.
 */
typedef struct NvExifPropertiesRec
{
    /** Software version of the application, in a 
        null terminated character string of length 32. */
    NvU8 SoftwareVersion[32];
    /** Make of the image, in null terminated character
        string of length 16. */
    NvU8 Make[16];
    /** Model of the image. */
    NvU8 Model[32];
    /** Digitized/original date and time, in null terminated
       character string of length 20. This field is set by the
       Dshow encoder using the current local time (as per zone set).
       The same value is stored for all 3 exif tags.
       \c DateTime \c DateTimeDigitized and \c DateTimeOriginal.
       Applications can ignore this field.	*/
    NvU8 DateTime[20];
} NvExifProperties;

/**
 * Structure for setting GPS (geo-tagging) information in the encoder
 * filter. Set using the ::ENCODER_GPS_INFO property on NVIDIA
 * Video Encoder Filter configured for JPEG encoding.
 */
typedef struct
{
    /** Specifies the size of the \c NvGpsInfo structure.  */
    NvU32 StructSize;
    /** An unsigned integer. When the version is 2.0.0.0, the value
    is to be specified as 0x02000000.  */
    NvU32 GPSVersionID;
    /** Null terminated character string of length 2 that specifies
    whether the latitude is North or South. 'N' specifies North and 'S'
    specifies South latitude.  */
    char GPSLatitudeRef[2];
    /** GPSLatitude is expressed as 3 rationales giving degrees, minutes, seconds
        respectively. GPSLatitudeNumerator and GPSLatitudeDenominator are both
        array's of 3 unsigned integers. For example, to express the value
        120 12 8, fill in the arrays as:
        - GPSLatitudeNumerator[0] = 240 and GPSLatitudeDenominator[0] = 2
        - GPSLatitudeNumerator[1] = 24 and GPSLatitudeDenominator[1] = 2
        - GPSLatitudeNumerator[2] = 16 and GPSLatitudeDenominator[2] = 2  */
    NvU32 GPSLatitudeNumerator[3];
    /** @see GPSLatitudeNumerator. */
    NvU32 GPSLatitudeDenominator[3];
    /** Null terminated character string of length 2 that specifies
    whether the longitude is East or West. 'W' specifies West and 'E' specifies
    East longitude.  */
    char GPSLongitudeRef[2];
    /** GPSLongitude is expressed as 3 rationales giving degrees, minutes, seconds
        respectively. GPSLongitudeNumerator and GPSLongitudeDenominator are both
        array's of 3 unsigned integers. For example, to express the value
        180 15 6, fill in the arrays as:
        - GPSLongitudeNumerator [0] = 360 and GPSLongitudeDenominator [0] = 2
        - GPSLongitudeNumerator [1] = 45 and GPSLongitudeDenominator [1] = 3
        - GPSLongitudeNumerator [2] = 12 and GPSLongitudeDenominator [2] = 2  */
    NvU32 GPSLongitudeNumerator[3];
    /** @see GPSLongitudeNumerator. */
    NvU32 GPSLongitudeDenominator[3];
    /** One byte character field.  */
    NvU8  GPSAltitudeRef;
    /** GPSLatitude is expressed as 1 rationale as follows: 
        <pre>     GPSAltitudeNumerator/GPSAltitudeDenominator    </pre>  */
    NvU32 GPSAltitudeNumerator;
    /** @see GPSAltitudeNumerator. */
    NvU32 GPSAltitudeDenominator;
    /** GPSTime is expressed as 3 rationales giving hour, minute and second.
        GPSTimeStampNumerator and GPSTimeStampDenominator are both arrays
        of 3 unsigned inteegers. For example, to express the GPSTime as
        5 20 15, fill in the arrays as:
        - GPSTimeStampNumerator [0] = 20 & GPSTimeStampDenominator [0] = 4
        - GPSTimeStampNumerator [1] = 40 & GPSTimeStampDenominator [1] = 2
        - GPSTimeStampNumerator [2] = 45 & GPSTimeStampDenominator [2] = 3  */
    NvU32 GPSTimeStampNumerator[3];
    /** @see GPSTimeStampNumerator. */
    NvU32 GPSTimeStampDenominator[3];
    /** Null terminated character string of length 32 used to specify
    the GPS Satellites used for measurements.  */
    char GPSSatellites[32];
    /** Null terminated character string of lenth 2 to specify the status
    of GPS reciever when the image was recorded. 'A' means the measurement is
    in progress, and 'V' means the measurement is interoperability.  */
    char GPSStatus[2];
    /** Null terminated character string of length 2 to specify the
    GPS measurement mode. 2 specifies 2D measurement and 3 specifies
    3D measurement.  */
    char GPSMeasureMode[2];
    /** Used to specify the GPS DOP (data degree of precision). It is expressed
        as rationale, hence the two fields for numerator and denominator.  */
    NvU32 GPSDOPNumerator;
    /** @see GPSDOPNumerator  */
    NvU32 GPSDOPDenominator;
    /** Null terminated character string of length 32 to specify geodetic
    survey data used by the GPS receiver. For example to specify if survey data
    is restricted to JAPAN, then specify TOKYO or WGS-84.  */
    char GPSMapDatum[32];
    /** Null terminated character string of length 11 used to express the date
    as YYYY:MM:DD.  */
    char GPSDateStamp[11];
    /** Null terminated character string of length 32.  */
    char GPSProcessingMethod[32];
} NvGpsInfo;

/**
 * Structure for thumbnail parameters that can be set.
 * Applications can use this structure to keep thumbnail related
 * information.
 */
typedef struct NvThumbnailPropertiesRec
{
    /** Embedded thumbnail width. */
    NvU32 ThumbnailWidth;
    /** Embedded thumbnail height. */
    NvU32 ThumbnailHeight;
    /* The thumbnail quality level. */
    NvJPEGThumbnailQuality ThumbnailQuality;
} NvThumbnailProperties;

/**
 * Structure that holds focus region;
 * Used by ::NvPropertyId_AutoFocusRegion property ID when used at
 * the pin level.
 */
typedef NvRect NvAutoFocusRegion;

/**
 * Structure that holds normalized focus region.
 * Used by ::NvPropertyId_AutoFocusRegion property ID when used at
 * the filter level.
 */
typedef NvRectF32 NvAutoFocusRegionNormalized;

/**
 * Structure for edge enhancement (sharpness) settings.
 * Used with the ::NvPropertyId_EdgeEnhancement property ID.
 * Edge enhancement increases the apparent sharpness of full
 * resolution images. It has no effect on downscaled images.
 */
typedef struct NvEdgeEnhanceSettingsRec
{
    /** Enable or disable edge enhancement. */
    NvBool EdgeEnhancement;
    /** Relative strength of edge enhancement to apply;
     *  range is 0 to 100. */
    NvF32 EdgeEnhanceStrength;
} NvEdgeEnhanceSettings;

/**
 * Structure for continuous I-Frame image details.
 * Set using the ::ENCODER_VIDEO_CONTINUOUS_IFRAME_PROPERTIES
 * property on NVIDIA Video Encoder Filter. To enable
 * continuous I-Frame mode, use
 * ::ENCODER_VIDEO_CONTINUOUS_IFRAME_MODE. When continuous I-Frame
 * mode is enabled and \a pImage is NULL, then the encoder filter
 * will keep sending the last frame repeatedly. In this case,
 * \a ImageInfoHeader is ignored. When \a pImage points to raw RGB565
 * or raw YUV420 (YV12) frame data, then the encoder filter will
 * use this image and deliver the same image repeatedly. In this
 * case, \a ImageInfoHeader contains information about the image
 * pointed to by \a pImage. When continous I-Frame mode is enabled,
 * every frame is encoded as an I-Frame.
 */
typedef struct NvContinuousIFrameSettingsRec
{
    /** Struct to hold the image details */
    BITMAPINFOHEADER ImageInfoHeader;
    /** Pointer to the image data */
    NvU8 *pImage;
} NvContinuousIFrameSettings;

/**
 * Structure for repeated frame image details.
 * Set using the ::NvPropertyId_RepeatFrame property on NVIDIA
 * Video Capture Filter. When \a RepeatedFrameMode is set to NV_TRUE
 * and \a pImage is NULL, then the capture filter will keep sending
 * the last frame repeatedly. In this case, \a ImageInfoHeader is
 * ignored. When \a pImage points to raw RGB565 or raw YUV420 (YV12)
 * frame data then the capture filter will use this image and
 * deliver the same image repeatedly. In this case,
 * \a ImageInfoHeader contains information about the image pointed to
 * by \a pImage.
 */
typedef struct NvRepeatedFrameSettingsRec
{
    /** Struct to hold the image details. */
    BITMAPINFOHEADER ImageInfoHeader;
    /** Pointer to the image data. */
    NvU8 *pImage;
    /** Flag to signal status of mode. */
    NvBool RepeatedFrameMode;
} NvRepeatedFrameSettings;

/**
 * Structure for record automatic gain control (AGC) properties.
 * Set using the ::NvPropertyId_RecordAGC property on NVIDIA
 * Audio Capture Filter. Set \a Enable to NV_TRUE to enable AGC or
 * to NV_FALSE to disable.
 */
typedef struct NvRecordAgcPropertiesRec
{
    /** NV_TRUE to enable; NV_FALSE to disable. */
    NvBool Enable;

    /**
     * Noise gate threshold in decibels.
     * Range : 
     * -90dB <= NoiseGateThreshold <= LowerCompressionThreshold.
     */
    NvF32 NoiseGateThreshold;

    /**
     * Lower compression threshold in decibels.
     * Range : 
     * NoiseGateThreshold <= LowerCompressionThreshold <= UpperCompressionThreshold.
     */
    NvF32 LowerCompressionThreshold;

    /**
     * Upper compression threshold in decibels.
     * Range : 
     * LowerCompressionThreshold <= UpperCompressionThreshold <= ClipTh.
     */
    NvF32 UpperCompressionThreshold;

    /**
     * Clipping threshold in decibels.
     * Range : 
     * UpperCompressionThreshold <= ClippingThreshold <= 0dB.
     */
    NvF32 ClippingThreshold;
} NvRecordAgcProperties;

/**
 * Defines the structure for setting range.
 * This structure is used by ::NvPropertyId_WhiteBalanceCCTRange and
 * ::NvPropertyId_FrameRateRange property on NVIDIA Audio Capture Filter.
 */
typedef struct NvCaptureRangeRec
{
    NvS32 Low;
    NvS32 High;
} NvCaptureRange;

/**
 * Defines the structure for getting the camera profile: preview, still capture,
 * and live capture. This structure is used by ::NvPropertyId_CameraProfile
 * property on the NVIDIA Video Capture Filter.
 *
 * @par Enabling and disabling profiling:
 * @code
 * NvCaptureProfile CaptureProfile;
 * NvOsMemset(&CaptureProfile, 0, sizeof(NvCaptureProfile));
 * CaptureProfile.Enable = NV_TRUE;     // Set to NV_FALSE to disable
 * pVideoSrcPropSet->lpVtbl->Set(
 *     pVideoSrcPropSet,
 *     &PROPSET_NvFilter,
 *     NvPropertyId_CameraProfile,
 *     NULL,
 *     0,
 *     (NvU32 *)&CaptureProfile,
 *     sizeof(NvCaptureProfile));
 * @endcode
 * @note Profiling must be enabled before calling Run on the filter graph
 *
 * @par Getting profiling values
 * @code
 * NvCaptureProfile CaptureProfile;
 * NvOsMemset(&CaptureProfile, 0, sizeof(NvCaptureProfile));
 * pVideoSrcPropSet->lpVtbl->Get(
 *     pVideoSrcPropSet,
 *     &PROPSET_NvFilter,
 *     NvPropertyId_CameraProfile,
 *     NULL,
 *     0,
 *     (NvU32 *)&CaptureProfile,
 *     sizeof(NvCaptureProfile),
 *     NULL);
 * if (CaptureProfile.TSLiveCaptureEnd > CaptureProfile.TSLiveCaptureStart)
 * {
 *     NvU64 CaptureFrameCount = CaptureProfile.LiveCaptureEndFrameCount -
 *         CaptureProfile.LiveCaptureStartFrameCount;
 *
 *     NvF32 CaptureFrameRate = (CaptureFrameCount * 10000000.0f) /
 *         (CaptureProfile.TSLiveCaptureEnd -
 *         CaptureProfile.TSLiveCaptureStart);
 *
 *     NvOsPrintf("Frame rate: %2.4f", CaptureFrameRate);
 * }
 * @endcode
 * @note Stop on the filter graph must be called before getting profiling
 * information.
 *
 * The above code demonstrates how to calculate video capture frame rate from
 * profiling information. Similarly, applications can also calculate still and
 * preview frame rates.
 */
typedef struct NvCaptureProfileRec
{
    /**
     * To enable, set this value to NV_TRUE and set using
     * ::NvPropertyId_CameraProfile before calling Run on the filter graph.
     * Set to NV_FALSE to disable profiling. NV_FALSE is the default.
     */
    NvBool Enable; 

    /** Time stamp when preview starts. The value is in 100ns. */
    NvU64  TSPreviewStart;

    /** Time stamp when preview stops. The value is in 100ns. */
    NvU64  TSPreviewEnd;

    /** The frame count when preview starts. */
    NvU64  PreviewStartFrameCount;

    /** The frame count when preview stops. */
    NvU64  PreviewEndFrameCount;

    /** Time stamp when still capture starts. The value is in 100ns. */
    NvU64  TSStillCaptureStart;

    /** Time stamp when still capture stops. The value is in 100ns. */
    NvU64  TSStillCaptureEnd;

    /** The frame count when still capture is triggered. */
    NvU64  StillCaptureStartFrameCount;

    /** Frame count when the last frame of the current burst is delivered. */
    NvU64  StillCaptureEndFrameCount;

    /** Time stamp when video capture starts. The valus is in 100ns. */
    NvU64  TSLiveCaptureStart;

    /** Time stamp when video capture stops. The value is in 100ns. */
    NvU64  TSLiveCaptureEnd;

    /** Frame count when video capture starts. */
    NvU64  LiveCaptureStartFrameCount;

    /** Frame count when video capture stops. */
    NvU64  LiveCaptureEndFrameCount;

    /** Exposure Time */
    NvS32  ExposureTime;

    /** Exposure ISO value */
    NvS32  ExposureISO;

    /** Time to deliver first preview frame after EnablePreview. */
    NvU64 TSFirstPreviewFrameAfterEnablePreview;
} NvCaptureProfile;

/**
 * MEDIASUBTYPE_YV12 Bits per Pixel (12)
 */
#define YV12_BITCOUNT 12

/**
 * MEDIASUBTYPE_RGB56 Bits per Pixel (16)
 */
#define RGB565_BITCOUNT 16

/**
 * MEDIASUBTYPE_BAYER
 */
#define BAYER_BITCOUNT 24

/**
 * FOURCC code for YV12 = 0x32315659
 */
#define FOURCC_YV12 MAKEFOURCC('Y', 'V', '1', '2')

/**
 * FOURCC code for BAYER = 0x
 */
#define FOURCC_BAYER MAKEFOURCC('B', 'A', '8', '1')

/** @name   Encoder Properties
 * These properties are the identifiers for properties set on
 * NVIDIA Video Encoder Filter and NVIDIA Audio Encoder Filter.
 * These properties are supported by both encoder DirectShow filters
 * and their DMO equivalents.
 * 
 * The video and audio encoder filter expose the @b IPropertyBag
 * interface that is used to set and get encoder properties.
 * For more information, see MSDN:
 * <a href="http://msdn.microsoft.com/en-us/library/aa908373.aspx"
 * target="_blank">http://msdn.microsoft.com/en-us/library/aa908373.aspx</a>
 *
 * All properties can be set after the video or audio encoder filter is
 * created. Some properties can also be changed at runtime.
 *
 * @par Example:
 * The following example shows how to set properties on NVIDIA Video
 * Encoder Filter. A similar method is used to set properties on the
 * NVIDIA Audio Encoder Filter.
 *
 * @code
 *
 * // QueryInterface for IPropertyBag from IBaseFilter interface or any
 * // interface exposed on the encoder filter.
 * pEncBaseFilter->QueryInterface(IID_IPropertyBag, (void **)pVidEncPropBag);
 *
 * // Use IPropertyBag::Write method to set a property. In this example, we
 * // will set the max bit-rate of NVIDIA Video Encoder Filter.
 * VARIANT var;
 * VariaintInit(&var);
 * var.vt = VT_I4;
 * var.lVal = 4000000;      // 4 Mbps converted by bps.
 * pVidEncPropBag->Write(ENCODER_VIDEO_MAX_BIT_RATE, &var);
 * VariantClear(&var);
 *
 * @endcode
 */
/*@{*/

/**
 * Forwards or rejects all samples.
 * Variant type for this property is VT_BOOL. This property is
 * common to NVIDIA Video and Audio Encoder filters. If set to
 * NV_TRUE then the encoder will reject all samples.
 * 
 * For example if an application wants to create a video-only file,
 * then the best way is not to render the audio portion of the
 * chain. Another option is provided by this property where the
 * application can render the audio portion of the chain but
 * ask NVIDIA Audio Encoder Filter to reject all audio data by
 * setting this property to NV_TRUE.
 */
static const WCHAR *ENCODER_REJECT_INPUT_SAMPLES =
    L"_YES";

/**
 * Audio encoder property to set format type.
 * This property is used to select
 * the type of encoding format. Variant type of this property is
 * VT_I4 and variant value (lVal) is set to the ::NvAudioCodecType
 * enumeration. This property is supported by NVIDIA Audio
 * Capture Filter.
 *
 * @par Supports:
 * AAC-LC, AAC+, E-AAC+, AMR-NB, AMR-WB audio encoding formats. 
 *
 * @par Note:
 * - Default audio encoder format type is AAC-LC (::NvAudioCodecType_AAC).
 *
 * @pre This property should be set before connection.
 *
 * @see NvAudioCodecType
 */
static const WCHAR *ENCODER_AUDIO_FORMAT_TYPE =
    L"NVAUDIO";

/**
 * Audio encoder property to set the maximum bit rate.
 * Variant type for this property is VT_I4. This property is used
 * to set maximum bit rate to be used for audio encoding. This
 * property is supported by NVIDIA Audio Encoder Filter.
 * 
 * Supported bit rates for AAC-LC, AAC+ and E-AAC+ varies
 * depending on sampling rate and number of channels. AMR-NB and
 * AMR-WB supports specific bit-rates only.
 *
 * @par AAC-LC (Generally used values):
 * - 8KHz, 11.025KHz, 16KHz - 32Kbps
 * - 16KHz, 22.05KHz, 24KHz - 64Kbps
 * - 32KHz, 44.1KHz, 48KHz - 128Kbps
 *
 * @par AAC+/E-AAC+:
 * - 64Kbps
 *
 * @par AMR-NB (bps):
 * - 4750, 5150, 5900, 6700, 7400, 10200, 12200
 *
 * @par AMR-WB (bps): 
 * - 7000, 9000, 12000, 14000, 16000, 18000, 20000, 23000, 24000
 *
 * @sa ENCODER_AUDIO_FORMAT_TYPE
 */
static const WCHAR *ENCODER_AUDIO_MAX_BIT_RATE =
    L"AMR_SPEECH_MODE";

/**
 * Video encoder property to set format type.
 * Variant type for this property is VT_I4. NVIDIA Video Encoder
 * Filter supports H.263, H.264 and MPEG-4 video encoding formats.
 * NVIDIA Video Encoder Filter also supports JPEG encoding.
 * This property is used to select the type of encoding format.
 * Set the variant value (lVal) to the NvVideoCodecType
 * enumeration.
 *
 * @par Note:
 * - Default video encoder format type is H.264 (::NvVideoCodecType_H264).
 *
 * @pre This property should be set before connection.
 *
 * @see         NvVideoCodecType
 */
static const WCHAR *ENCODER_VIDEO_FORMAT_TYPE =
    L"NVVIDEO";

/**
 * Video encoder property to set maximum bit rate.
 * Variant type for this property is VT_I4. This property is used
 * to set maximum bit rate to be used for video encoding.
 * Supported bit rates range from 128 Kbps to up to 15 Mbps based on
 * the video resolution.
 * This property can be dynamically set at runtime (dynamic bitrate).
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_VIDEO_MAX_BIT_RATE =
    L"MAX_BIT_RATE";

/**
 * Video encoder property to set video app type.
 * Variant type for this property is VT_I4. Set the variant value
 * (lVal) to the ::NvAppType enumeration.
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_VIDEO_APP_TYPE =
    L"VIDEO_APP_TYPE";

/**
 * Video encoder property to set video quality level.
 * Variant type for this property is VT_I4. Set the variant value
 * (lVal) to the ::NvQualityLevel enumeration.
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_VIDEO_QUALITY_LEVEL =
    L"VIDEO_QUALITY_LEVEL";

/**
 * Jpeg encoder property to set Jpeg quality level.
 * Variant type for this property is VT_I4. Set the variant value
 * (lVal) to the ::NvJpegQualityLevel enumeration.
 */
static const WCHAR *ENCODER_JPEG_QUALITY_LEVEL =
    L"ENCODER_JPEG_QUALITY_LEVEL";

/**
 * Video encoder property to set video codec level within the
 * supported profile.
 * Variant type for this property is VT_I4. Set the variant value
 * (lVal) to either a value from the ::NvH264EncLevelIdc enumeration
 * for H.264 or a value from the ::NvMPEG4EncLevelIdc enumeration for
 * MPEG-4. For H.263 and JPEG, this value is ignored. Set variant value
 * ::NV_VIDEO_CODEC_LEVEL_UNDEF to let the encoder automatically
 * select a level.
 */
static const WCHAR *ENCODER_VIDEO_CODEC_LEVEL =
    L"VIDEO_CODEC_LEVEL";

/**
 * Video encoder property to set error resiliency level.
 * Variant type for this property is VT_I4. Set the variant value
 * (lVal) to a value from the ::NvErrorResiliencyLevel enumeration.
 * This property is applicable only when the application type
 * (ENCODER_VIDEO_APP_TYPE) is set to ::NvAppType_VideoTelephony.
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_VIDEO_ERROR_RESILIENCY_LEVEL =
    L"VIDEO_ERROR_RESILIENCY_LEVEL";

/**
 * Video encoder property to set intra frame interval.
 * Variant type for this property is VT_I4. Set the variant value
 * (lVal) to the desired intra frame interval.
 */
static const WCHAR *ENCODER_VIDEO_INTRA_FRAME_INTERVAL =
    L"VIDEO_INTRA_FRAME_INTERVAL";

/**
 * Video (JPEG) encoder property to set JPEG encoded size.
 * Variant type for this property is VT_I4. Sets to the value in
 * bytes under which the JPEG encoder will attempt to encode
 * frames it receives from the capture filter. This property is
 * available only when video format (::ENCODER_VIDEO_FORMAT_TYPE) is
 * set to JPEG.
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_VIDEO_JPEG_ENCODED_SIZE =
    L"JPEG_ENCODED_SIZE";

/**
 * Video encoder property to insert I-Frame on client request.
 * Variant type for this property is VT_BOOL. Variant value is
 * ignored. When called, NVIDIA Video Encoder Filter will
 * encode the next frame as I-Frame.
 * This property can be dynamically set at runtime.
 */
static const WCHAR *ENCODER_VIDEO_IFRAME_INSERT =
    L"VIDEO_IFRAME_INSERT";

/**
 * Video encoder property to enable VBR rate control.
 * Variant type for this property is VT_BOOL. If this property is 
 * set to NV_FALSE (0), the CBR Rate Control Algorithm is enforced, and 
 * CBR is also the default.
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_VIDEO_ENABLE_VBR_RATE_CONTROL =
    L"VIDEO_ENABLE_VBR_RATE_CONTROL";

/**
 * Video (JPEG) encoder property to insert thumbnail.
 * Variant type for this property is VT_BOOL. When set to NV_TRUE,
 * NVIDIA Video Encoder Filter will embed thumbnail along with
 * JPEG image. This property is supported only when video format
 * (::ENCODER_VIDEO_FORMAT_TYPE) is set to JPEG.
 */
static const WCHAR *ENCODER_INSERT_THUMBNAIL =
    L"INSERT_THUMBNAIL";

/**
 * Video (JPEG) encoder property to set thumbnail width.
 * Variant type for this property is VT_I4. Specifies the width
 * of thumbnail to embed in JPEG image. This property is supported
 * only when video format (::ENCODER_VIDEO_FORMAT_TYPE) is set to
 * JPEG. To enable thumbnail use the ::ENCODER_INSERT_THUMBNAIL
 * property.
 */
static const WCHAR *ENCODER_THUMBNAIL_WIDTH =
    L"THUMBNAIL_WIDTH";

/**
 * Video (JPEG) encoder property to set thumbnail height
 * Variant type for this property is VT_I4. Specifies the height
 * of thumbnail to embed in JPEG image. This property is supported
 * only when video format (::ENCODER_VIDEO_FORMAT_TYPE) is set to
 * JPEG. To enable thumbnail use the ::ENCODER_INSERT_THUMBNAIL
 * property.
 */
static const WCHAR *ENCODER_THUMBNAIL_HEIGHT =
    L"THUMBNAIL_HEIGHT";

/**
 * Video (JPEG) encoder property to set thumbnail quality.
 * Variant type for this property is VT_I4. Specifies the quality
 * of thumbnail to embed in JPEG image. Variant value is one of
 * the values from the ::NvJPEGThumbnailQuality enumeration. This
 * property is supported only when video format
 * (::ENCODER_VIDEO_FORMAT_TYPE) is set to JPEG.
 */
static const WCHAR *ENCODER_THUMBNAIL_QUALITY =
    L"THUMBNAIL_QUALITY";

/**
 * Video (JPEG) encoder property to set Exif properties
 * Variant type for this property is VT_BYREF. Property value
 * is pointer to the ::NvExifProperties structure. This property is
 * supported only when video format (::ENCODER_VIDEO_FORMAT_TYPE) is
 * set to JPEG.
 */
static const WCHAR *ENCODER_EXIF_PROPERTIES =
    L"EXIF_PROPERTIES";

/**
 * Video (JPEG) encoder properties to set GPS properties. Variant type
 * for this property is VT_BYREF. Property value is a pointer to the
 * ::NvGpsInfo structure. This property is supported only when video format
 * (::ENCODER_VIDEO_FORMAT_TYPE) is set to JPEG.
 */
static const WCHAR *ENCODER_GPS_INFO =
    L"GPS_INFO";

/**
 * Video (JPEG) encoder property to set/reset Exif programming.
 * Variant type for this property is VT_BOOL. Set to NV_TRUE to
 * embed Exif in JPEG image. 
 * @pre This property is supported only when
 * video format (::ENCODER_VIDEO_FORMAT_TYPE) is set to JPEG.
 */
static const WCHAR *ENCODER_PROGRAM_EXIF =
    L"PROGRAM_EXIF";

/**
 * Video (JPEG) encoder property to set Exif orientation.
 * Variant type for this property is VT_I4. Set to appropriate value 
 * as defined in ::NvExifOrientation.
 * @pre This property is supported only when
 * video format (::ENCODER_VIDEO_FORMAT_TYPE) is set to JPEG.
 * @pre ::ENCODER_PROGRAM_EXIF must be set prior to setting this
 * property for JPEG encoder.
 */
static const WCHAR *ENCODER_PROGRAM_EXIF_ORIENTATION =
    L"PROGRAM_EXIF_ORIENTATION";

/**
 * Video encoder property to set/unset continuous I-Frame
 * insertion mode.
 * Variant type for this property is VT_BOOL. Set to NV_TRUE to
 * enable continuous I-Frame mode. To configure continous I-Frame
 * mode use ::ENCODER_VIDEO_CONTINUOUS_IFRAME_PROPERTIES.
 */
static const WCHAR *ENCODER_VIDEO_CONTINUOUS_IFRAME_MODE =
    L"VIDEO_CONTINUOUS_IFRAME_MODE";

/**
 * MPEG-4 video encoder property to set/unset Mona compliance.
 * Variant type for this property is VT_BOOL. Set to NV_TRUE to
 * enable encoding compliance to the Mona specification.
 * Because MONA is supported only for MPEG-4 encoder, applications must set 
 * ::ENCODER_VIDEO_FORMAT_TYPE to ::NvVideoCodecType_MPEG4 prior to setting
 * ::ENCODER_VIDEO_MPEG4_MONA_COMPLIANCE to NV_TRUE. The only resolution
 * is QCIF, i.e. 176x144, per Mona. Therefore applications must
 * program source to capture QCIF and program encoder accordingly.
 * Bitrate for encoding is forced to be 64 Kbps, and this is non configurable.
 *
 * @par To get Mona-compliant MPEG-4 bitstreams:
 *
 * -# Set the \c ENCODER_VIDEO_FORMAT_TYPE to \c NvVideoCodecType_MPEG4.
 * -# Set the source capture width and thus video encoder width to 176.
 * -# Set the source capture height and thus video encoder height to 144.
 * -# Set the \c ENCODER_VIDEO_MPEG4_MONA_COMPLIANCE attribute to NV_TRUE.
 * 
 * When the \c ENCODER_VIDEO_MPEG4_MONA_COMPLIANCE attribute is set for
 * MPEG-4 encoder, the following settings are required by Mona to be the default
 * and are not user configurable:
 *  
 * - profileAndLevel = 8
 * - object = 1
 * - TimestampResCount = 30000; 
 * - DataPartition = ENABLE; 
 * - Reprocess = ENABLE; 
 * - NumOfDataPartitions = 2; 
 * - RVLC : DISABLED;
 * - DecoderConfigurationInformation:
 * - Base64: AAABsAgAAAG1CQAAAQAAAAEgAIRdTCgsIJCijw==
 * - Hex: 00-00-01-b0-08-00-00-01-b5-09-00-00-01-00-00-00-01-20-
 *        00-84-5d-4c-28-2c-20-90-a2-8f
 */
static const WCHAR *ENCODER_VIDEO_MPEG4_MONA_COMPLIANCE =
    L"VIDEO_MPEG4_MONA_COMPLIANCE";

/**
 * Video encoder property to set video temporal trade off.
 * Variant type for this property is VT_I4. Set to a value from
 * the the ::NvVideoTemporalTradeOffLevel enumeration.
 * This property can be dynamically set at runtime.
 */
static const WCHAR *ENCODER_VIDEO_TEMPORAL_TRADE_OFF =
    L"VIDEO_TEMPORAL_TRADE_OFF";

/**
 * Video encoder property to set image properties for I-Frame
 * insertion mode.
 * Variant type for this property is VT_BYREF. Variant value
 * is a pointer to the ::NvContinuousIFrameSettings structure.
 * To enable continous I-Frame mode, use the
 * ::ENCODER_VIDEO_CONTINUOUS_IFRAME_MODE property.
 */
static const WCHAR *ENCODER_VIDEO_CONTINUOUS_IFRAME_PROPERTIES =
    L"VIDEO_CONTINUOUS_IFRAME_PROPERTIES";

/**
 * Video encoder property to set packet based encoding properties.
 * Variant type for this property is VT_BYREF. Variant value is
 * one of the value from the ::NvVideoEncPacketType enumeration.
 */
static const WCHAR *ENCODER_VIDEO_ENC_PACKET =
    L"VIDEO_ENC_PACKET";

/**
 * Video encoder property to set percentage of Intra-MB.
 * Variant type for this property is VT_I4. Variant value is
 * a number that specified percentage of Intra-MB.
 * This property can be dynamically set at runtime.
 */
static const WCHAR *ENCODER_VIDEO_ENC_PERCENTAGE_INTRA_MB =
    L"VIDEO_ENC_PERCENTAGE_INTRA_MB";

/**
 * Audio encoder property to set audio file format.
 * Variant type for this property is VT_I4. Variant value is a
 * value from the ::NvAudioAmrNBFormat enumeration when audio format
 * is set to AMR-NB and is a value from the ::NvAudioAmrWBFormat
 * enumeration when audio format is set to AMR-WB. To set the
 * audio format use ::ENCODER_AUDIO_FORMAT_TYPE.
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_AUDIO_FILE_FORMAT =
    L"AUDIO_FILE_FORMAT";

/**
 * Audio encoder property to set voice activity detection (VAD).
 * Variant type for this property is VT_BOOL. Set to NV_TRUE to
 * enable VAD. This property is supported by NVIDIA Audio
 * Encoder Filter.
 *
 * @pre This property should be set before connection.
 */
static const WCHAR *ENCODER_AUDIO_VAD_ENABLE =
    L"AUDIO_VAD_ENABLE";

/**
 * Video encoder read-only property to retrieve decoder config info (DCI).
 * Variant type for this property is VT_BYREF. Variant value is a pointer to
 * ::NvVideoEncDecodeConfigInfo structure.
 * NVIDIA Video Encoder Filter allocates sizeof(NvVideoEncDecodeConfigInfo)
 * memory using CoTaskMemAlloc() and returns the pointer in the byref member of
 * the VARIANT structure. The application must ensure that it frees this
 * memory using CoTaskMemFree().
 */
static const WCHAR *ENCODER_VIDEO_ENC_DECODER_CONFIG_INFO =
    L"VIDEO_ENC_DECODER_CONFIG_INFO";

/**
 * Video Encoder property to set the jpeg capture rotation angle.
 * Variant type for this property is VT_I4. Valid values can be:
 * - NvDdk2dTransform_None
 * - NvDdk2dTransform_Rotate90
 * - NvDdk2dTransform_Rotate180
 * - NvDdk2dTransform_Rotate270
 *
 * The JPEG image created will be rotated by the given angle
 * and dimension of image will also change accordingly.
 * @par Example:
 * If capture is of 320 x 240, and rotation angle
 * is \c NvDdk2dTransform_Rotate90, then the JPEG image created will be
 * of 240 x 320.
 */
static const WCHAR *ENCODER_JPEG_CAPTURE_ROTATION_ANGLE = 
    L"JPEG_CAPTURE_ROTATION_ANGLE";

/**
 * Video Encoder property to set a bitmap to merge on input frames. Variant
 * type for this property type is VT_BYREF and property value is a pointer to
 * ::NvBitmapDesc structure. This property can be used to merge date and time
 * on JPEG images. If rotation is also specified using the
 * ::ENCODER_JPEG_CAPTURE_ROTATION_ANGLE property then the bitmap is merged
 * into input frames after it is rotated. To stop applying the overlay to
 * frames, set this property again with property value set to NULL.
 * The pBmpData member of ::NvBitmapDesc must be allocated and destroyed by
 * the application. The application can delete pBmpData after setting this
 * property. Video Encoder filter creates a copy of the set bitmap data. Only
 * 32bpp bitmaps are supported. This property is supported only when
 * ENCODER_VIDEO_FORMAT_TYPE is set to NvVideoCodecType_JPEG.
 */
static const WCHAR *ENCODER_VIDEO_MERGE_BITMAP =
    L"ENCODER_VIDEO_MERGE_BITMAP";

/*@}*/

/**
 * Defines types of audio codec output formats.
 * Used with the ::ENCODER_AUDIO_FORMAT_TYPE property and set on the
 * NVIDIA Audio Encoder Filter, which supports the following audio encoded
 * output types:
 * - AMR-NB (\c NvAudioCodecType_NBAMR) Compliant Speech Encoded Bitstream.
 * - AMR-WB (\c NvAudioCodecType_WBAMR) Compliant Speech Encoded Bitstream.
 * - AAC-LC (\c NvAudioCodecType_AAC) Complaint Audio Encoded Bitstream.
 * - AAC-Plus (\c NvAudioCodecType_AACPLUS) Compliant Audio Encoded Bitstream.
 * - EAAC (\c NvAudioCodecType_HEAAC) Compliant Audio Encoded Bitstream.
 *
 * The following definitions are currently unsupported/unused by 
 * NVIDIA Audio Encoder Filter:
 * - \c NvAudioCodecType_MP3
 * - \c NvAudioCodecType_AACLTP
 */
typedef enum
{
    /** Sets audio codec type to AMR-NB. */
    NvAudioCodecType_NBAMR = 1,
    /** Sets audio codec type to AMR-WB. */
    NvAudioCodecType_WBAMR,
    /** Reserved. */
    NvAudioCodecType_MP3,
    /** Sets audio codec type to AAC. */
    NvAudioCodecType_AAC,
    /** Not used. */
    NvAudioCodecType_AACLTP,
    /** Sets audio codec type to AAC+. */
    NvAudioCodecType_AACPLUS,
    /** Sets audio codec type to E-AAC. */
    NvAudioCodecType_HEAAC,
    /** Not used. */
    NvAudioCodecType_None,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvAudioCodecType_Force32 = 0x7FFFFFFF
} NvAudioCodecType;

/**
 * Defines types of video codecs output formats.
 * Used with the ::ENCODER_VIDEO_FORMAT_TYPE property and set on
 * NVIDIA Video Encoder Filter, which supports the following
 * video/still encoded output types:
 * - MPEG4 (\c NvVideoCodecType_MPEG4) Compliant Simple Profile Bitstream.
 * - H264 (\c NvVideoCodecType_H264) Compliant Baseline Simple Profile Bitstream.
 * - H263/(MPEG4 SVH) (\c NvVideoCodecType_H263) Compliant Baseline Profile Bitstream.
 * - JPEG (\c NvVideoCodecType_JPEG) Compliant Bitstream for Still Image.
 */
typedef enum
{
    /** Sets video codec type to MPEG-4. */
    NvVideoCodecType_MPEG4 = 100,
    /** Sets video codec type to H.264. */
    NvVideoCodecType_H264,
    /** Sets video codec type to H.263. */
    NvVideoCodecType_H263,
    /** Sets video codec type to JPEG. */
    NvVideoCodecType_JPEG,
    /** Sets video codec type to Bayer. */
    NvVideoCodecType_Bayer,
    /** Not used. */
    NvVideoCodecType_None,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvVideoCodecType_Force32 = 0x7FFFFFFF
} NvVideoCodecType;

/**
 * Constant to set encoder codec level to auto.
 * Used with the ::ENCODER_VIDEO_CODEC_LEVEL video encoder property.
 */
typedef enum { NV_VIDEO_CODEC_LEVEL_UNDEF = 0xFF };

/**
 * H.264 levels for baseline profile.
 * Used with the ::ENCODER_VIDEO_CODEC_LEVEL video encoder property. Refer to 
 * H.264 standard documentation for details on H.264 profile and levels.
 * Please consider different level enforced restrictions offered for encoding
 * as per respective H.264 Standard w.r.t max frame rate, max bitrate and 
 * max resolution. Required frame rate can be controlled by setting video source
 * property ::NvPropertyId_FrameRateRange \c High and \c Low values
 * appropriately, For example, Level 1.0 in H.264 supports encoding of only QCIF 
 * resolution at 64 kbps and 15 fps. In that case, one must set source filter
 * property \c NvPropertyId_FrameRateRange \c High and \c Low values to 15 to 
 * achieve DirectShow recording at 15 fps, and set source capture resolution to
 * QCIF using IAMStreamConfig::SetFormat().
 */
typedef enum
{
    /** Sets H.264 encoder level to 1.0. */
    NvH264EncLevelIdc_10 = 10,
    /** Sets H.264 encoder level to 1b. */
    NvH264EncLevelIdc_1B,
    /** Sets H.264 encoder level to 1.1. */
    NvH264EncLevelIdc_11,
    /** Sets H.264 encoder level to 1.2. */
    NvH264EncLevelIdc_12,
    /** Sets H.264 encoder level to 1.3. */
    NvH264EncLevelIdc_13,
    /** Sets H.264 encoder level to 2.0. */
    NvH264EncLevelIdc_20,
    /** Sets H.264 encoder level to 2.1. */
    NvH264EncLevelIdc_21,
    /** Sets H.264 encoder level to 2.2. */
    NvH264EncLevelIdc_22,
    /** Sets H.264 encoder level to 3.0. */
    NvH264EncLevelIdc_30,
    /** Sets H.264 encoder level to 3.1. */
    NvH264EncLevelIdc_31,
    /** Sets H.264 encoder level to 3.2. */
    NvH264EncLevelIdc_32,
    /** Sets H.264 encoder level to 4.0. */
    NvH264EncLevelIdc_40,
    /** Sets H.264 encoder level to 4.1. */
    NvH264EncLevelIdc_41,
    /** Sets H.264 encoder level to 4.2. */
    NvH264EncLevelIdc_42,
    /** Sets H.264 encoder level to 5.0. */
    NvH264EncLevelIdc_50,
    /** Sets H.264 encoder level to 5.1. */
    NvH264EncLevelIdc_51,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvH264EncLevelIdc_Force32 = 0x7FFFFFFF
} NvH264EncLevelIdc;

/**
 * MPEG-4 levels for simple profile.
 * Used with the ::ENCODER_VIDEO_CODEC_LEVEL video encoder property. Refer to
 * MPEG-4 standard document for details on MPEG-4 profile and levels.
 * Please consider different level enforced restrictions offered for encoding
 * as per respective MPEG-4 Standard w.r.t max frame rate, max bitrate, and 
 * max resolution permissible. Required frame rate can be controlled by setting
 * video source property ::NvPropertyId_FrameRateRange \c High and \c Low values
 * appropriately, For example, Level 0 in MPEG4 supports encoding of only QCIF
 * resolution at 64 kbps and 15 fps. In that case, one must set source filter
 * property \c NvPropertyId_FrameRateRange \c High and \c Low values to 15 to
 * achieve DirectShow recording at 15 fps and set source capture resolution to
 * QCIF using IAMStreamConfig::SetFormat().
 */
typedef enum
{
    /** Sets MPEG4 encoder level to 0 */
    NvMPEG4EncLevelIdc_0 = 0,
    /** Sets MPEG4 encoder level to 1 */
    NvMPEG4EncLevelIdc_1,
    /** Sets MPEG4 encoder level to 2 */
    NvMPEG4EncLevelIdc_2,
    /** Sets MPEG4 encoder level to 3 */
    NvMPEG4EncLevelIdc_3,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvMPEG4EncLevelIdc_Force32 = 0x7FFFFFFF
} NvMPEG4EncLevelIdc;

/**
 * Video temporal trade-off levels.
 * Used with the ::ENCODER_VIDEO_TEMPORAL_TRADE_OFF encoder property.
 */
typedef enum
{
    /** Encoder delivers frames as fast as possible. */
    NvVideoTemporalTradeOffLevel_None = 0x0,
    /** Encoder delivers frames at 24 FPS. */
    NvVideoTemporalTradeOffLevel_1,
    /** Encoder delivers frames at 20 FPS. */
    NvVideoTemporalTradeOffLevel_2,
    /** Encoder delivers frames at 15 FPS. */
    NvVideoTemporalTradeOffLevel_3,
    /** Encoder delivers frames at 10 FPS. */
    NvVideoTemporalTradeOffLevel_4,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvVideoTemporalTradeOffLevel_Force32 = 0x7FFFFFFF
} NvVideoTemporalTradeOffLevel;

/**
 * Defines application type.
 * This enumeration is used by both, NVIDIA Video Capture fitler and NVIDIA
 * Video Encoder Filter. To set on the video capture filter, use the
 * ::NvPropertyId_AppType property ID. To set on the video encoder property,
 * use ::ENCODER_VIDEO_APP_TYPE.
 */
typedef enum
{
    /**
     * Sets application type to camcorder.
     * The video capture filter will halt timestamps when the capture stream
     * or filter is paused.
     * The video encoder filter parameters are:
     * - Intra-frame after application specified interval.
     * - Follows closed-GOP rate control.
     * - Packetization type is macro-block based.
     * - No intra-referesh pattern included.
     * - FMO is not supported.
     * - De-blocking filtering is done across slices.
     * - Contraint intra-prediction is off.
     */
    NvAppType_Camcorder = 1,

    /**
     * Sets application type to video telephony.
     * The video capture filter will continue generating timestamps when
     * the capture stream or filter is paused.
     * The video encoder filter parameters are:
     * - Only first frame is intra-frame.
     * - Follows open-GOP rate control.
     * - When error resiliency level is set to None then packetization is
     *   macro-block based otherwise it is bit based.
     * - Has intra-refresh pattern.
     * - FMO is supported.
     * - De-blocking filtering is done across slices only when error
     *   resiliency level is set to None. Otherwise it is done inside slices.
     * - Constraint intra-prediction is off when error resiliency level is set
     *   None. Otherwise it is on.
     */
    NvAppType_VideoTelephony,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvAppType_Force32 = 0x7FFFFFFF
} NvAppType;

/**
 * Defines encoder quality levels.
 * Used with ::ENCODER_VIDEO_QUALITY_LEVEL video encoder property.
 */
typedef enum
{
    /**
     * Sets video encoding quality to low.
     * H.264:
     * - Motion estimation mode decision is done after inter-pel.
     * - IP decision is using only Intra16x16 mode.
     * - Only 1 search start point is used for motion estimation.
     * MPEG4 and H.263:
     * - 1 motion vector per macro-block.
     * - Half-pel motion estimation disabled.
     * - Motion estimation mode decision is done after integer-pel.
     * - AC/DC prediction off.
     */
    NvQualityLevel_Low = 1,

    /**
     * Sets video encoding quality to medium.
     * H.264:
     * - Motion estimation mode decision is done after half-pel.
     * - IP decision is using only Intra16x16 mode.
     * - 2 search points are used for motion estimation.
     * MPEG4 and H.263:
     * - Up to 4 motion vectors per macro-block.
     * - Half-pel motion estimation disabled.
     * - Motion estimation mode decision is done after integer pel.
     * - AC/DC prediction is dynamically on.
     * - H.263 standard restriction - AC/DC prediction off.
     */
    NvQualityLevel_Medium,

    /**
     * Sets video encoding quality to high.
     * H.264:
     * - Motion estimation mode decision is done after quarter-pel.
     * - IP decision uses Intra16x16 as well as Intra4x4 mode.
     * - 2 search start points are used for motion estimation.
     * MPEG4 and H.263:
     * - Up to 4 motion vectors per macro-block.
     * - Half-pel motion estimation enabled.
     * - Motion estimation mode decision is done after half-pel.
     * - AC/DC prediction is dynamically on.
     * - H.263 standard restriction - AC/DC prediction off.
     */
    NvQualityLevel_High,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvQualityLevel_Force32 = 0x7FFFFFFF
} NvQualityLevel;
/**
 * Defines the JPEG encoder quality levels.
 * Used with ::ENCODER_JPEG_QUALITY_LEVEL video encoder property.
 */
typedef enum
{
    NvJpegQualityLevel_VeryLow = 1,
    NvJpegQualityLevel_Low,
    NvJpegQualityLevel_Medium,
    NvJpegQualityLevel_High,
    NvJpegQualityLevel_VeryHigh,
    NvJpegQualityLevel_Force32 = 0x7FFFFFFF
} NvJpegQualityLevel;
/**
 * Defines error resiliency levels.
 * Used with ::ENCODER_VIDEO_ERROR_RESILIENCY_LEVEL video encoder
 * property.
 */
typedef enum
{
    /**
     * Sets video encoder error resiliency level to none.
     * H.264
     * - Packetization is macro-block based.
     * - De-blocking across slides.
     * - Constraint intra prediction is off.
     * - No intra-refresh.
     * - No FMO.
     * MPEG4 and H.263:
     * - Packetization in fixed bits. 
     * - H.263 standard restriction - MB based GOBs.
     * - Data partition disabled.
     * - RVLC disabled.
     * - HEC disabled.
     * - Intra refresh disabled.
     */
    NvErrorResiliencyLevel_None = 1,

    /**
     * Sets video encoder error resiliency level to low.
     * H264:
     * - Packetization is bit-based.
     * - De-blocking within slices.
     * - Contraint intra prediction is on.
     * - Intra-refresh is enabled.
     * - FMO supported.
     * MPEG4 and H.263:
     * - Packetization in fixed bits.
     * - H.263 standard restriction - MB based GOBs.
     * - Data partition disabled.
     * - RVLC disabled.
     * - HEC disabled.
     * - Intra refresh enabled.
     */
    NvErrorResiliencyLevel_Low,

    /**
     * Sets video encoder error resiliency level to high.
     * H.264
     * (Encoder attributes same as ::NvErrorResiliencyLevel_Low)
     * MPEG4 and H.263:
     * - Packetization in fixed bits.
     * - H.263 standard restriction- MB based GOBs.
     * - Data partition enabled.
     * - RVLC enabled.
     * - HEC marker after every resynchronization marker.
     * - Intra refresh enabled.
     */
    NvErrorResiliencyLevel_High,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvErrorResiliencyLevel_Force32 = 0x7FFFFFFF
} NvErrorResiliencyLevel;

/**
 * Device ID to select between MicIn and LineIn.
 * @note This feature is currently not used.
 */
enum
{
    MICIN_DEVICE_ID = 0,
    LINEIN_DEVICE_ID = 1
};

/**
 * Defines video encoder packet types.
 * Used with the ::ENCODER_VIDEO_ENC_PACKET video encoder property
 * via the ::NvVideoEncPacket structure.
 * Packet type is not supported in H.263 encoding by standard.
 */
typedef enum
{
    /** Video encoder type is set to macro-block based. */
    NvVideoEncPacketType_MBLK,
    /** Video encoder type is set to bit based. */
    NvVideoEncPacketType_BITS,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvVideoEncPacketType_Force32 = 0x7FFFFFFF
} NvVideoEncPacketType;

/**
 * Properties for packet based encoding.
 * Used with the ::ENCODER_VIDEO_ENC_PACKET video encoder property.
 */
typedef struct
{
    /** Packet type can be either in macroblocks or bits. */
    NvVideoEncPacketType PacketType;

    /**
     * Length of packet (packet size) depending upon its type. The minimum
     * packet size is 1 when PacketType is ::NvVideoEncPacketType_MBLK and is 8
     * when PacketType is ::NvVideoEncPacketType_BITS. The packet is
     * automatically clipped at frame end if packet size is specified more
     * than the encoded frame length.
     */
    NvU32 PacketLength;
} NvVideoEncPacket;

/**
 * Defines audio encoder file formats for AMR-WB.
 * Used with the ::ENCODER_AUDIO_FILE_FORMAT audio encoder property.
 * ::NvAudioAmrWBFormat_MIME is the default.
 */
typedef enum
{
    /** AMR-WB format is set to default. */
    NvAudioAmrWBFormat_Default,
    /** AMR-WB format is set to ITU. */
    NvAudioAmrWBFormat_ITU,
    /** AMR-WB format is set to MIME. */
    NvAudioAmrWBFormat_MIME,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvAudioAmrWBFormat_Force32 = 0x7FFFFFFF
} NvAudioAmrWBFormat;

/**
 * Defines audio encoder formats for AMR-NB.
 * Used with the ::ENCODER_AUDIO_FILE_FORMAT audio encoder property.
 * ::NvAudioAmrNBFormat_MMS is the default.
 */
typedef enum
{
    /** AMR-NB format is set to MMS. */
    NvAudioAmrNBFormat_MMS,
    /** AMR-NB format is set to IF1. */
    NvAudioAmrNBFormat_IF1,
    /** AMR-NB format is set to IF2. */
    NvAudioAmrNBFormat_IF2,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvAudioAmrNBFormat_Force32 = 0x7FFFFFFF
} NvAudioAmrNBFormat;

/**
 * Defines auto focus functions.
 * @note This enumeration is not used.
 */
typedef enum
{
    /** Auto focus - manual. */
    NvFocusControl_On = 0,
    /** Auto focus - automatic. */
    NvFocusControl_Auto,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvFocusControl_Force32 = 0x7FFFFFFF
} NvFocusControl;

/**
 * Defines camera metering modes.
 * This property is set on the video capture filter using the
 * ::NvPropertyId_CameraMeteringMode property ID. Metering-mode controls how
 * the ISP computes the proper exposure time and gain for the scene. This
 * setting has no effect if exposure is specified manually.
 */
typedef enum
{
    /**
     * Will compute the scene exposure using a small central region of the
     * image.
     */
    NvCameraMeteringMode_Spot,

    /**
     * Computes scene exposure using the central region of the image. 
     */
    NvCameraMeteringMode_Center,

    /**
     * Computes the scene exposure using the full image.
     */
    NvCameraMeteringMode_FullFrame,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvCameraMeteringMode_Force32 = 0x7FFFFFFF
}NvCameraMeteringMode;

/**
 * Defines Exif orientation values for JPEG.
 */
typedef enum
{
    /// Row_0 - visual top,      Col_0 - visual left.
    NvExifOrientation_0_Degrees          = 1,
    /// Row_0 - visual top,      Col_0 - visual right.
    NvExifOrientation_0_Degrees_Flip_H   = 2,
    /// Row_0 - visual bottom,   Col_0 - visual right.
    NvExifOrientation_180_Degrees        = 3,
    /// Row_0 - visual bottom,   Col_0 - visual left.
    NvExifOrientation_180_Degrees_Flip_H = 4,
    /// Row_0 - visual left,     Col_0 - visual top.
    NvExifOrientation_270_Degrees_Flip_H = 5,
    /// Row_0 - visual right,    Col_0 - visual top.
    NvExifOrientation_270_Degrees        = 6,
    /// Row_0 - visual right,    Col_0 - visual bottom.
    NvExifOrientation_90_Degrees_Flip_H  = 7,
    /// Row_0 - visual left,     Col_0 - visual bottom.
    NvExifOrientation_90_Degrees         = 8,
    /// Ignore -- Forces compilers to make 32-bit enums.
    NvExifOrientation_Force16            = 0x7FFF

}NvExifOrientation;

/**
* Types of userdata
*/
typedef enum{
    Nv3GPWriterUserDataType_Album =1,                      /**< Album title and track number. */
    Nv3GPWriterUserDataType_Author,                          /**< Media author name. */
    Nv3GPWriterUserDataType_Classification,               /**< Media classification. */
    Nv3GPWriterUserDataType_Copyright,                     /**< Copyright, etc. */
    Nv3GPWriterUserDataType_Description,                  /**< Media description. */
    Nv3GPWriterUserDataType_Genre,                           /**< Media genre. */
    Nv3GPWriterUserDataType_Keywords,                     /**< Media keywords. */
    Nv3GPWriterUserDataType_LocationInfo,                /**< Media location info. */
    Nv3GPWriterUserDataType_PerfName,                    /**< Media performer name. */
    Nv3GPWriterUserDataType_Rating,                          /**< Media rating. */
    Nv3GPWriterUserDataType_Title,                             /**< Media title. */
    Nv3GPWriterUserDataType_Year,                             /**< Year when media was recorded. */
    Nv3GPWriterUserDataType_UserSpecific,                /**< Users own box type, Xtra, etc. */               
    /// Ignore -- Forces compilers to make 32-bit enums.
    Nv3GPWriterUserDataType_Force32 = 0x7FFFFFFF        
}Nv3GPWriterUserDataType;

typedef enum { NV_MAX_ATOM_SIZE = 5 };

/**
  * Data type can be any of enum type in ::NvMM3GPWriterUserDataTypes.
  * <pre>
  * "usrBoxType" is the name of the box type
  *     =======================  
  *      Name               |      Box Type
  *     =======================
  *      Album              |      albm
  *      Author             |      auth
  *      Classification     |      clsf
  *      copyright          |      cprt
  *      Description        |      dscp
  *      Genre              |      gnre
  *      KeyWords           |      kywd
  *      Location Info      |      loci
  *      Performer          |      perf
  *      Rating             |      rtng
  *      title              |      titl
  *      year               |      yrrc
  *      user specific      |      < Any 4 bytes type as user needs>
  *     =================================
  * "sizeOfUserData" is size of total data in this box
  * "pUserData" pointer to the buffer holding user data that 
  *  needs to be added to udta
  * </pre>
  */
typedef struct NvWriterUserDataConfig
{
    Nv3GPWriterUserDataType dataType;
    NvU8 usrBoxType[NV_MAX_ATOM_SIZE];
    NvU32 sizeOfUserData;
    NvU8 *pUserData; 
} NvWriterUserDataConfig;

/**
 * General struture to simulate 3x3 matrix.
 */
typedef struct Nv3x3DshowMatrixRec
{
    /// First row of 3x3 matrix.
    NvF32 m00, m01, m02;
    /// Second row of 3x3 matrix.
    NvF32 m10, m11, m12;
    /// Third row of 3x3 matrix.
    NvF32 m20, m21, m22;
} Nv3x3DshowMatrix;

/**
 * Structure to get decode config information (DCI) from NVIDIA Video Encoder 
 * filter. To retrieve DCI, use the ::ENCODER_VIDEO_ENC_DECODER_CONFIG_INFO
 * property name constant.
 */
typedef struct NvVideoEncDecodeConfigInfoRec
{
    /** Size of the structure. Set to sizeof(NvVideoEncDecodeConfigInfo). */
    NvU32 StructSize;
    /** Decode configuration information. */
    NvU8 DecodeConfigInfo[NV_VIDEOENC_DCI_SIZE];
    /** Actual size of \c DecodeConfigInfo. */
    NvU8 SizeOfDecodeConfigInfo;
} NvVideoEncDecodeConfigInfo;

/**
 * Structure that describes a bitmap. This structure is used to provide a
 * bitmap using the ::ENCODER_VIDEO_MERGE_BITMAP property.
 */
typedef struct NvBitmapDescRec
{
    /** Position at which the bitmap will be merged on the input frame */
    NvPoint Position;
    /** The actual bitmap */
    BITMAP bmp;
    /** NV_TRUE if bitmap must be flipped during merging */
    NvBool FlipVertical;
} NvBitmapDesc;

/**
 * Custom info from the imager.
 */
typedef struct NvCaptureCustomInfoRec
{
    NvU32 SizeOfData;
    NvU8  *pData;
} NvCaptureCustomInfo;

/** @} */
#endif /* INCLUDED_NV_DMO_H */
