/*
 * Copyright (c) 2008 NVIDIA Corporation.  All Rights Reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property and
 * proprietary rights in and to this software and related documentation.  Any
 * use, reproduction, disclosure or distribution of this software and related
 * documentation without an express license agreement from NVIDIA Corporation
 * is strictly prohibited.
 */

#ifndef INCLUDED_CAPTURE_ENCODE_SAMPLE_H
#define INCLUDED_CAPTURE_ENCODE_SAMPLE_H

#include <stdio.h>
#include <strmif.h>
#include <uuids.h>
#include <control.h>
#include <initguid.h>
//#include "nvos.h"
//#include "DWORD.h"
#include <amvideo.h>
//#include "nvdshow_decode.h"
//#include "nvdshow_encode.h"
//#include "nvdshow_common.h"
//#include "nvencode_test_defines.h"

/**
 * @brief       This is the main function of this test. This test is a simple
 *              Application to encode Captured video using H264 codec and Audio 
 *              as per AAC Encoding. The Video Recording happens in Camcorder mode
 *              for 720p with medium quality and no error resiliency setting.
 *              Default Bitrate for Video Encoding is considered to be 10 Mbps,
 *              while for Audio Sampling rate for AAC is assumed to be 44.1 kHz.
 *              This application receives only time in sec for recording as an 
 *              command line parameter.
 * @param[in]   Handle to NvTestApplication rec.
 */
//void NvEncodeTest(NvTestApplicationHandle h);

/**
 * @brief       Sets all the registry entries, essentially sets registry
 *              setting for Output File Format which is 3GP and Sets 
 *              FrameRate at which Dshow Filter Graph is running (30 FPS).
 * @returns     DWORD values
 * @retval      DWORD_Success if all steps go ok
 * @retval      DWORD_NotSupported if any of the steps gives an error
 */
HRESULT SetRegProperties(void);

/**
 * @brief       Render video into the graph. Meaning, Creates instance of 
 *              Nvidia Video Capture Filter as Video Source Filter, 
 *              Initialises preview for D1 Resolution,
 *              Adds Nvidia H264 Video Encoder Filter to the thus formed graph.
 * @param[in]   pGraph Pointer to the filter graph
 * @param[in]   pCaptureGraphBuilder Pointer to the capture graph builder
 * @param[in]   pSinkFilter Pointer to IBaseFilter of the writer filter
 * @returns     A standard HRESULT values
 * @retval      S_OK if all steps go ok
 * @retval      E_FAIL if any of the steps gives an error
 */
HRESULT
RenderVideoInGraph(
    IGraphBuilder *pGraph,
    ICaptureGraphBuilder2 *pCaptureGraphBuilder,
    IBaseFilter *pSinkFilterStill, IBaseFilter *pSinkFilterVideo);

/**
 * @brief       Render audio into the graph. Meaning, Creates instance of
 *              Nvideo Audio Capture Filter (MIC-IN) as Audio source Filter,
 *              Adds Nvidia AAC Audio Encoder Filter to the thus formed graph.
 * @param[in]   pGraph Pointer to the filter graph
 * @param[in]   pCaptureGraphBuilder Pointer to the capture graph builder
 * @param[in]   pSinkFilter Pointer to IBaseFilter of the writer filter
 * @returns     A standard HRESULT values
 * @retval      S_OK if all steps go ok
 * @retval      E_FAIL if any of the steps gives an error
 */
HRESULT
RenderAudioInGraph(
    IGraphBuilder *pGraph,
    ICaptureGraphBuilder2 *pCaptureGraphBuilder,
    IBaseFilter *pSinkFilter);

/**
 * @brief       Add Sink Filter. Sink Filter here is 3GP Muxer which also has
 *              file write capabilities.
 * @param[in]   pGraph Pointer to the filter graph
 * @param[out]  ppSinkFilter Returns pointer to writer filter
 * @returns     A standard HRESULT values
 * @retval      S_OK if all steps go ok
 * @retval      E_FAIL if any of the steps gives an error
 */
HRESULT AddSinkFilter(IGraphBuilder *pGraph, IBaseFilter **ppSinkFilter, BOOL bIsStill);

/**
 * @brief       Runs the graph
 * @param[in]   pGraph Pointer to the filter graph
 * @returns     A standard HRESULT values
 * @retval      S_OK if all steps go ok
 * @retval      E_FAIL if any of the steps gives an error
 */
HRESULT RunGraph(HWND ghApp, IGraphBuilder *pGraph);

/**
 * @brief       Controls the pins of the capture filter
 * @param[in]   pGraph          Graph pointer used to render the output pin
 * @param[in]   PinCategory     Pin category to be controlled
 * @param[in]   pStart          Start time
 * @param[in]   pStop           Stop time
 * @returns     S_OK if successful; an error code otherwise
 */
HRESULT
ControlCaptureFilterPin(
    IGraphBuilder *pGraph,
    const GUID PinCategory,
    REFERENCE_TIME *pStart,
    REFERENCE_TIME *pStop);

/**
 * @brief       Sets the file format at ouput in registry settings.
 * @remarks     The extension of the output file is decided on
 *              this type
 * @param[in]   FileFormat      The format of the file to generate
 * @returns     DWORD         Indicates sucess or failure
 */
//DWORD NvSetFileFormat(NvFileFormat FileFormat);

/**
 * @brief       Sets the frame rate at which the DMO chain is
 *              running
 * @param[in]   VideoFrameRate  Frame rate of the video
 * @returns     DWORD         Indicates success or failure
 */
DWORD NvSetMxFrameRate(WORD VideoFrameRate);

/**
 * @brief       Sets Property Bag variants for Video Encoder Filter.
 *              Eg. Codec type, Quality and Error Resiliency  Level, etc.
 * @param[in]   pEncoderFilter pointer to video encoder filter
 * @returns     HRESULT values
 * @retval      S_OK if all variants set
 * @retval      E_FAIL if error
 */
HRESULT SetVideoProperties(IBaseFilter *pEncoderFilter);
HRESULT SetVideoPropertiesH264(IBaseFilter *pEncoderFilter);
HRESULT SetVideoPropertiesJPEG(IBaseFilter *pEncoderFilter);

/**
 * @brief       Sets Property Bag variants for Audio
 *              Eg. Audio Codec Type, Sampling rate etc.
 * @param[in]   pEncoderFilter pointer to audio encoder filter
 * @returns     HRESULT values
 * @retval      S_OK if all variants set
 * @retval      E_FAIL if error
 */
HRESULT SetAudioProperties(IBaseFilter *pEncoderFilter);

/**
 * @brief       Returns NV_TRUE if the OS version is Windows CE
 * @returns     NV_TRUE if OS is Windows CE; NV_FALSE otherwise
 */
//NvBool IsOsVersionWinCE(void);

/**
 * @brief       Returns IKsPropertySet pointer corresponding to the pins
 *              of the source filter
 * @param[in]   pSourceFilter       Source filter
 * @param[in]   pPreviewMediaType   Pointer to preview pin's media type
 * @param[in]   pCaptureMediaType   Pointer to capture pin's media type
 * @returns     HRESULT standard values
 */
HRESULT
SetSourceFilterPinsMediaType(
    IBaseFilter *pSourceFilter,
    const AM_MEDIA_TYPE *pPreviewMediaType,
    const AM_MEDIA_TYPE *pCaptureMediaType);

#endif  // INCLUDED_ENCODE_TEST_H
