/*
 * Copyright (c) 2008 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA Corporation is strictly prohibited.
 */

#include "sensor_common.h"
//#include "sensor_plugin.h"
//#include "nvassert.h" 
//#include "imager_util.h"
#include "nvodm_imager.h"
//#include "nvodm_imager_device.h"
#include "nvodm_imager_int.h"
#include "imager_hal.h" //shard ressources with driver
//#include "imager_util.h"

#include "GPIOLib.h"
#include "I2CLib.h"

#define I2C_ADDRESS					(0x21)

/****************************************************************************
 * Implementing for a New Sensor:
 * 
 * When implementing for a new sensor from this template, there are four 
 * phases:
 * Phase 0: Gather sensor information. In phase 0, we should gather the sensor
 *          information for bringup such as sensor capabilities, set mode i2c
 *          write sequence, NvOdmIoAddress in nvodm_query_discovery_imager.h,
 *          and so on.
 * Phase 1: Bring up a sensor. After phase 1, we should be able to do still
 *          captures. Since this is a yuv sensor, there won't be many
 *          interactions between camera and sensor.
 * Phase 2: Fully functioning sensor. After phase 2, the sensor should be fully
 *          functioning as described in nvodm_imager.h
 *
 * To help implementing for a new sensor, the template code will be marked as
 * Phase 0, 1, 2
 ****************************************************************************/



/**
 * ----------------------------------------------------------
 *  Start of Phase 0 code
 * ----------------------------------------------------------
 */

/**
 * Sensor Specific Variables/Defines. Phase 0.
 *
 * If a new sensor code is created from this template, the following
 * variable/defines should be created for the new sensor specifically.
 * 1. A sensor capabilities (NvOdmImagerCapabilities) for this specific sensor
 * 2. A sensor set mode sequence list  (an array of SensorSetModeSequence)
 *    consisting of pairs of a sensor mode and corresponding set mode sequence
 * 3. A set mode sequence for each mode defined in 2.
 */

/**
 * Sensor yuv's capabilities. Phase 0. Sensor Dependent.
 * This is the first thing the camera driver requests. The information here
 * is used to setup the proper interface code (CSI, VIP, pixel type), start
 * up the clocks at the proper speeds, and so forth.
 * For Phase 0, one could start with these below as an example for a yuv
 * sensor and just update clocks from what is given in the reference documents
 * for that device. The FAE may have specified the clocks when giving the
 * reccommended i2c settings.
 */

static NvOdmImagerCapabilities g_SensorYuvCaps =
{
    "max9526",  // string identifier, used for debugging
    NvOdmImagerSensorInterface_Parallel8, // interconnect type
    {
//ITU_R BT.656 (default) stream contains 4:2:2 data with optional embedded synchronization codes
        NvOdmImagerPixelType_UYVY, //NvOdmImagerPixelType_VYUY,//NvOdmImagerPixelType_UYVY,
    },
    NvOdmImagerOrientation_0_Degrees, // rotation with respect to main display
    NvOdmImagerDirection_Away,          // toward or away the main display
    27000, // initial sensor clock (in khz)
//VGA (640 x 480): 15 fps at 12 MHz (default), programmable up to 30 fps at 27 MHz
    { {27000, 2.0}, {0,0} }, // preferred clock profile
    {
//check
        NvOdmImagerSyncEdge_Falling, // hsync polarity
//check
        NvOdmImagerSyncEdge_Falling, // vsync polarity
        NV_FALSE,                   // uses mclk on vgp0
    }, // parallel settings (VIP)
    {0,0,0,NV_FALSE,0}, // serial settings (CSI) (not used)
    {0, 0}, // min blank time, shouldn't need adjustment
    0, // preferred mode, which resolution to use on startup
    0, // FOCUSER_GUID, only set if focuser code available
    0, // FLASH_GUID, only set if flash device code is available
    NVODM_IMAGER_CAPABILITIES_END
};

/**
 * A list of sensor mode and its SetModeSequence. Phase 0. Sensor Dependent.
 * When SetMode is called, it searches the list for desired resolution and
 * frame rate and write the SetModeSequence through i2c.
 */


static SensorSetModeSequence g_SensorYuvSetModeSequenceList[] =
{
	//DAN not sure what x, y is
     // WxH        x, y    fps   set mode sequence
//check
    //{{{534, 72}, {720-534, (480/2)-72},  50, 0}, NULL, NULL},
    //{{{(720-10)-10, (480/2)-10}, {10, 11},  30, 0}, NULL, NULL},
	//{{{720, 272}, {0, 0},  30, 0}, NULL, NULL},
	//{{{640/2, 480/2}, {0, 0},  15, 0}, NULL, NULL},
	{{{720, (480/2)}, {0, 0},  15, 1}, NULL, NULL},


};

/**
 * ----------------------------------------------------------
 *  Start of Phase 1 code
 * ----------------------------------------------------------
 */


static BOOL I2CSetup()
{
	BOOL Success;
	BYTE data[20];
	//DWORD VerMaj,VerMin,VerBuild;
	//I2cIntGetLibVersion(&VerMaj, &VerMin, &VerBuild);
	//wprintf(L"Basic Setup for MAX9526\r\n");
	//wprintf(L"Library Version %d.%d Build %d\r\n\r\n", VerMaj, VerMin, VerBuild);

	if (I2CInit())
	{
		
		Success = GetI2CLock(500);
		//I2CSetSpeed(I2C_100KBPS);				// Currently, this sets LastError=1210 on Tegra

		//I2CBurstRead(I2C_ADDRESS, data,0x11,1);
		//I2CBurstRead(I2C_ADDRESS, data,0x10,1);
		
		data[0]=0x80;  //RESET
		I2CBurstWrite(I2C_ADDRESS, data, 0x0F, 1);																																																																																																																																																																												        
		Sleep(10);

		data[0]=0xCC;
		I2CBurstWrite(I2C_ADDRESS, data, 0x01, 1);

		data[0]=0x00;  //NSEL = CVBS in on AIN1
		I2CBurstWrite(I2C_ADDRESS, data, 0x00, 1);

		////*********************************
		//data[0]=0x1A;  
		//I2CBurstWrite(I2C_ADDRESS, data, 0x31, 1);
		//data[0]=0x81;  
		//I2CBurstWrite(I2C_ADDRESS, data, 0x32, 1);
		//data[0]=0x84;  //Polarities
		//I2CBurstWrite(I2C_ADDRESS, data, 0x33, 1);
		//data[0]=00;  
		//I2CBurstWrite(I2C_ADDRESS, data, 0x34, 1);
		//data[0]=0x00;  
		//I2CBurstWrite(I2C_ADDRESS, data, 0x35, 1);
		//data[0]=0x7D;  //Polarities
		//I2CBurstWrite(I2C_ADDRESS, data, 0x36, 1);
		//data[0]=0xA1;  //Polarities
		//I2CBurstWrite(I2C_ADDRESS, data, 0x37, 1);
		//data[0]=0x41;  //Polarities
		//I2CBurstWrite(I2C_ADDRESS, data, 0xE8, 1);
		//data[0]=0x84;  //Polarities
		//I2CBurstWrite(I2C_ADDRESS, data, 0xE9, 1);
		//data[0]=0x06;  //Polarities
		//I2CBurstWrite(I2C_ADDRESS, data, 0xEA, 1);
		////*********************************

		//data[0]=0x57;  //Enable SFL
		//I2CBurstWrite(I2C_ADDRESS, data, 0x03, 1);
	

		//data[0]=0x57;  //Enable SFL
		//I2CBurstWrite(I2C_ADDRESS, data, 0x04, 1);
	
		//data[0]=0x41;  //Select SH1
		//I2CBurstWrite(I2C_ADDRESS, data, 0x14, 1);

		//data[0]=0x02;  //Clear NEWAV_MODE, SAV/EAV to suit ADV video encoders
		//I2CBurstWrite(I2C_ADDRESS, data, 0x31, 1);
		data[0]=0x1A;  
		I2CBurstWrite(I2C_ADDRESS, data, 0x31, 1);

		//data[0]=0x81;  
		//I2CBurstWrite(I2C_ADDRESS, data, 0x32, 1);


		data[0]=0xA2;  //MWE enable manual window, color kill threshold to 2
		I2CBurstWrite(I2C_ADDRESS, data, 0x3D, 1);

		data[0]=0x6A;  //BLM optimization
		I2CBurstWrite(I2C_ADDRESS, data, 0x3E, 1);

		data[0]=0xA0;  //BGB optimization
		I2CBurstWrite(I2C_ADDRESS, data, 0x3F, 1);

		//NEW DAN
		//data[0]=0xC4;  //V bit begin
		//I2CBurstWrite(I2C_ADDRESS, data, 0x33, 1);

		data[0]=0x01; //HS to Actrive Video
		I2CBurstWrite(I2C_ADDRESS, data, 0x34, 1);
		data[0]=29;
		I2CBurstWrite(I2C_ADDRESS, data, 0x35, 1);
		data[0]=0x01;  
		I2CBurstWrite(I2C_ADDRESS, data, 0x37, 1);


		data[0]=0x01;  
		I2CBurstWrite(I2C_ADDRESS, data, 0x58, 1);

		data[0]=0x80;  //Hidden space
		I2CBurstWrite(I2C_ADDRESS, data, 0x0E, 1);
		data[0]=0x81;  //ADC configuration
		I2CBurstWrite(I2C_ADDRESS, data, 0x55, 1);
		data[0]=0x00;  //User space
		I2CBurstWrite(I2C_ADDRESS, data, 0x0E, 1);

		//NEW DAN
		////data[0]=0x65;  //V bit begin
		////I2CBurstWrite(I2C_ADDRESS, data, 0xE5, 1);
		//data[0]=0x44;  //V bit begin (works for Handy CAM)
		//I2CBurstWrite(I2C_ADDRESS, data, 0xE6, 1);

		data[0]=0x54;  //V bit begin (works for cheap test cam)
		I2CBurstWrite(I2C_ADDRESS, data, 0xE9, 1);

		
		//Print Info for Debug only
		Sleep(60);
		I2CBurstRead(I2C_ADDRESS, data,0x11,1);
		NKDbgPrintfW(L"ADV7180 ID: 0x%x\n\r",data[0]);
		I2CBurstRead(I2C_ADDRESS, data,0x10,1);
		NKDbgPrintfW(L"ADV7180 Status 1: 0x%x\n\r",(0xff&data[0]));
		NKDbgPrintfW(L"ADV7180 Status 1 Autodedect: %d\n\r",(data[0]&0x70)>>4);
		I2CBurstRead(I2C_ADDRESS, data,0x12,1);
		NKDbgPrintfW(L"ADV7180 Status 2: 0x%x\n\r",(0xff&data[0]));
		I2CBurstRead(I2C_ADDRESS, data,0x13,1);
		NKDbgPrintfW(L"ADV7180 Status 3: 0x%x\n\r",(0xff&data[0]));
		ReleaseI2CLock();
	}
	I2CDeInit();

	return 1 ;
}


/**
 * Sensor yuv's private context. Phase 1.
 */
typedef struct SensorYuvContextRec
{
    // Phase 1 variables.
    //NvOdmImagerI2cConnection I2c;
    NvU32 ModeIndex;
    NvU32 NumModes;
    NvOdmImagerPowerLevel PowerLevel;
    //ImagerGpioConfig GpioConfig;

}SensorYuvContext;

/**
 * Static functions
 * Some compilers require these to be declared upfront.
 */
static NvBool SensorYuv_Open(NvOdmImagerHandle hImager);
static void SensorYuv_Close(NvOdmImagerHandle hImager);

static void
SensorYuv_GetCapabilities(
    NvOdmImagerHandle hImager,
    NvOdmImagerCapabilities *pCapabilities);

static void
SensorYuv_ListModes(
    NvOdmImagerHandle hImager,
    NvOdmImagerSensorMode *pModes,
    NvS32 *pNumberOfModes);
    
static NvBool
SensorYuv_SetMode(
    NvOdmImagerHandle hImager,
    const SetModeParameters *pParameters,
    NvOdmImagerSensorMode *pSelectedMode,
    SetModeParameters *pResult);

static NvBool
SensorYuv_SetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel PowerLevel);

static NvBool
SensorYuv_SetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    const void *pValue);

static NvBool
SensorYuv_GetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    void *pValue);

static void
SensorYuv_GetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel *pPowerLevel);


/**
 * SensorYuv_Open. Phase 1.
 * Initialize sensor yuv and its private context
 */
static NvBool SensorYuv_Open(NvOdmImagerHandle hImager)
{
    SensorYuvContext *pSensorYuvContext = NULL;

    if (!hImager || !hImager->pSensor)
        return NV_FALSE;

    pSensorYuvContext =
        (SensorYuvContext*)malloc(sizeof(SensorYuvContext));
    if (!pSensorYuvContext)
        goto fail;

    memset(pSensorYuvContext, 0, sizeof(SensorYuvContext));

	//Set GPIOs (for the moment just set the multiplexed ones the main pins are setup by the driver
	//for verfication you can use the GPIOconfig Tool)
	SetGPIOAltFn(TEGRA_GPIONUM('b',4),-1,DIR_IN);	//D7 Sodimm 59
	SetGPIOAltFn(TEGRA_GPIONUM('a',7),-1,DIR_IN);	//D6 Sodimm 67

	if(!I2CSetup())
		return FALSE;

    pSensorYuvContext->NumModes =
        NV_ARRAY_SIZE(g_SensorYuvSetModeSequenceList);
    pSensorYuvContext->ModeIndex =
        pSensorYuvContext->NumModes + 1; // invalid mode
    
    hImager->pSensor->pPrivateContext = pSensorYuvContext;

    return NV_TRUE;    

fail:
    free(pSensorYuvContext);
    return NV_FALSE;
}

/**
 * SensorYuv_Close. Phase 1.
 * Free the sensor's private context
 */
static void SensorYuv_Close(NvOdmImagerHandle hImager)
{
    SensorYuvContext *pContext = NULL;
        
    if (!hImager || !hImager->pSensor || !hImager->pSensor->pPrivateContext)
        return;

    pContext = (SensorYuvContext*)hImager->pSensor->pPrivateContext;

    // cleanup
    free(pContext);
    hImager->pSensor->pPrivateContext = NULL;
}

/**
 * SensorYuv_GetCapabilities. Phase 1.
 * Returnt sensor Yuv's capabilities
 */
static void
SensorYuv_GetCapabilities(
    NvOdmImagerHandle hImager,
    NvOdmImagerCapabilities *pCapabilities)
{
    // Copy the sensor caps from g_SensorYuvCaps
    memcpy(pCapabilities, &g_SensorYuvCaps,
        sizeof(NvOdmImagerCapabilities));
}

/**
 * SensorYuv_ListModes. Phase 1.
 * Return a list of modes that sensor yuv supports.
 * If called with a NULL ptr to pModes, then it just returns the count
 */
static void
SensorYuv_ListModes(
    NvOdmImagerHandle hImager,
    NvOdmImagerSensorMode *pModes,
    NvS32 *pNumberOfModes)
{
    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;
    NvS32 i;

    if (pNumberOfModes)
    {
        *pNumberOfModes = (NvS32)pContext->NumModes;
        if (pModes)
        {
            // Copy the modes from g_SensorYuvSetModeSequenceList
            for (i = 0; i < *pNumberOfModes; i++)
                pModes[i] = g_SensorYuvSetModeSequenceList[i].Mode;
        }
    }
}

/**
 * SensorYuv_SetMode. Phase 1.
 * Set sensor yuv to the mode of the desired resolution and frame rate.
 */
static NvBool
SensorYuv_SetMode(
    NvOdmImagerHandle hImager,
    const SetModeParameters *pParameters,
    NvOdmImagerSensorMode *pSelectedMode,
    SetModeParameters *pResult)
{
    NvBool Status = NV_FALSE;
    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;
    NvU32 Index;

    //ASSERT(pParameters);

    //NKDbgPrintfW("Setting resolution to %dx%d\n",
    //    pParameters->Resolution.width, pParameters->Resolution.height);

    // Find the right entrys in g_SensorYuvSetModeSequenceList that matches
    // the desired resolution and framerate
    for (Index = 0; Index < pContext->NumModes; Index++)
    {
        if ((pParameters->Resolution.width == g_SensorYuvSetModeSequenceList[Index].
             Mode.ActiveDimensions.width) &&
            (pParameters->Resolution.height == g_SensorYuvSetModeSequenceList[Index].
             Mode.ActiveDimensions.height))
            break;
    }

    // No match found
    if (Index == pContext->NumModes)
        return NV_FALSE;

    if (pSelectedMode)
    {
        *pSelectedMode = g_SensorYuvSetModeSequenceList[Index].Mode;
    }

    if (pContext->ModeIndex == Index)
        return NV_TRUE;

    // i2c writes for the set mode sequence
    //Status = WriteI2CSequence(&pContext->I2c,
    //             g_SensorYuvSetModeSequenceList[Index].pSequence);
    //if (!Status)
    //   return NV_FALSE;

    pContext->ModeIndex = Index;

    if (pResult)
    {
        pResult->Resolution = g_SensorYuvSetModeSequenceList[Index].
                              Mode.ActiveDimensions;
        pResult->Exposure = 0;
        memset(pResult->Gains, 0, sizeof(NvF32) * 4);
    }


    return Status;
}

/**
 * SensorYuv_SetPowerLevel. Phase 1
 * Set the sensor's power level
 */
static NvBool
SensorYuv_SetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel PowerLevel)
{
    NvBool Status = NV_TRUE;
    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;
    //const NvOdmPeripheralConnectivity *pConnections;

    if (pContext->PowerLevel == PowerLevel)
        return NV_TRUE;

    ////NV_ASSERT(hImager->pSensor->pConnections);
    //pConnections = hImager->pSensor->pConnections;

    switch(PowerLevel)
    {
        case NvOdmImagerPowerLevel_On:
/*
            Status = SetPowerLevelWithPeripheralConnectivity(pConnections,
                         &pContext->I2c, &pContext->GpioConfig,
                         NvOdmImagerPowerLevel_On);
            if (!Status)
            {
                NvOdmOsDebugPrintf("NvOdmImagerPowerLevel_On failed.\n");
                return NV_FALSE;
            }
*/
            break;

        case NvOdmImagerPowerLevel_Standby:
/*
            Status = SetPowerLevelWithPeripheralConnectivity(pConnections,
                         &pContext->I2c, &pContext->GpioConfig,
                         NvOdmImagerPowerLevel_Standby);
*/
            break;
        case NvOdmImagerPowerLevel_Off:
/*
            Status = SetPowerLevelWithPeripheralConnectivity(pConnections,
                         &pContext->I2c, &pContext->GpioConfig,
                         NvOdmImagerPowerLevel_Off);
            
            // wait for the standby 0.1 sec should be enough
            NvOdmOsWaitUS(100000); 
*/
            break;
        default:
//            //NV_ASSERT("!Unknown power level\n");
            Status = NV_FALSE;
            break;
    }

    if (Status)
        pContext->PowerLevel = PowerLevel;

    return Status;
}

/**
 * SensorYuv_SetParameter. Phase 2.
 * Set sensor specific parameters. 
 * For Phase 1. This can return NV_TRUE.
 */
static NvBool
SensorYuv_SetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    const void *pValue)
{
    switch(Param)
    {
        case NvOdmImagerParameter_SensorInputClock:
            if(SizeOfValue == sizeof(NvOdmImagerClockProfile))
            {
                // Verify the input clock is valid.
                NvU32 InClock =
                    ((NvOdmImagerClockProfile*)pValue)->ExternalClockKHz;

                if (InClock < g_SensorYuvCaps.InitialSensorClockRateKHz)
                    return NV_FALSE;
            }
            break;
        case NvOdmImagerParameter_OptimizeResolutionChange:
			if(SizeOfValue!=sizeof(NvBool)){break;}

            // Sensor yuv doesn't support optimized resolution change.
            if (*((NvBool*)pValue) == NV_FALSE)
                return NV_TRUE;
            else
                return NV_FALSE;

            break;

        case NvOdmImagerParameter_SelfTest:
            // not implemented.
            break;
        default:
//            //NV_ASSERT(!"Not Implemented");
            break;
    }


    return NV_TRUE;
}

/**
 * SensorYuv_GetParameter. Phase 1.
 * Get sensor specific parameters
 */
static NvBool
SensorYuv_GetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    void *pValue)
{

    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;

    switch(Param)
    {
        case NvOdmImagerParameter_DeviceStatus:
            if(SizeOfValue ==sizeof(NvOdmImagerDeviceStatus))
            {
                NvOdmImagerDeviceStatus *pStatus;
                pStatus = (NvOdmImagerDeviceStatus *)pValue;
/*
                {
                    NvOdmImagerI2cConnection *pI2c = &pContext->I2c;
                    // Pick your own useful registers to use for debug.
                    // If the camera hangs, these register values are printed
                    // Sensor Dependent.
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x0000);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[0]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2104);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[1]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2703);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[2]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2705);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[3]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2737);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[4]);

                    pStatus->Count = 5;
                }
*/
            }
            break;

        case NvOdmImagerParameter_SensorFrameRate:
			if(SizeOfValue == sizeof(NvF32)){

            // TODO: Get the real frame rate.
            *((NvF32*)pValue) = 30.0f;
            break;
			}

        default:
           //NV_ASSERT(!"Not Implemented");
           break;
    }

    return NV_TRUE;
}

/**
 * SensorYuv_GetPowerLevel. Phase 2.
 * Get the sensor's current power level
 */
static void
SensorYuv_GetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel *pPowerLevel)
{
    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;

    *pPowerLevel = pContext->PowerLevel;
}

/**
 * SensorYuv_GetHal. Phase 1.
 * return the hal functions associated with sensor yuv
 */
NvBool SensorPlugin_GetHal_YUV_ADV7180(NvOdmImagerHandle hImager)
{
    if (!hImager || !hImager->pSensor)
        return NV_FALSE;

    hImager->pSensor->pfnOpen = SensorYuv_Open;
    hImager->pSensor->pfnClose = SensorYuv_Close;
    hImager->pSensor->pfnGetCapabilities = SensorYuv_GetCapabilities;
    hImager->pSensor->pfnListModes = SensorYuv_ListModes;
    hImager->pSensor->pfnSetMode = SensorYuv_SetMode;
    hImager->pSensor->pfnSetPowerLevel = SensorYuv_SetPowerLevel;
    hImager->pSensor->pfnGetPowerLevel = SensorYuv_GetPowerLevel;
    hImager->pSensor->pfnSetParameter = SensorYuv_SetParameter;
    hImager->pSensor->pfnGetParameter = SensorYuv_GetParameter;

    return NV_TRUE;
}
