/*
 * Copyright (c) 2008 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA Corporation is strictly prohibited.
 */

// ..\..\..\..\libs\lib\ARMV4IDbg

//#include "sensor_plugin.h"
//#include "nvassert.h" 
//#include "imager_util.h"
#include "sensor_common.h"
#include "nvodm_imager.h"
//#include "nvodm_imager_device.h"
#include "nvodm_imager_int.h"
#include "imager_hal.h" //shard ressources with driver
//#include "imager_util.h"

#include "GPIOLib.h"
#include "I2CLib.h"


// ******* these resolution are available so far *****
// 320x240 15fps				preview working, capture working
#define USE_QVGA_MODE			1
// 640x480 15fps				preview working, capture working
#define USE_VGA_MODE			1
// 800x600 30fps				preview working, capture not
#define USE_SVGA_30_MODE	1
// 1280x720 15fps				preview shifted, capture working
#define USE_720P30_MODE		1
// 1280x960 30fps				preview working, capture working
#define USE_SXGA_30_MODE	1
// 1920x1080 3.75fps		preview working, capture not
#define USE_1080P375_MODE	0
// 1920x1080 15fps			preview shifted, capture not
#define USE_1080P15_MODE	1
// 1600x1200 15fps			preview working, capture not
#define USE_UXGA_MODE			1
// 1600x1200 15fps			preview working, capture not
#define USE_1600_1200_MODE	0
// 2592x1944 7.5fps			preview working, capture not
#define USE_5MP_75_MODE			1
// 2592x1944 3.75fps		preview working, capture not
#define USE_5MP_375_MODE		0


#define CAM_PWRDWN_PIN				168
#define CAM_RESET_PIN				170

#define CAM_RESET_POL				1
#define CAM_PWRDWN_POL				0

//For I2C
#define SEQUENCE_WAIT_MS                (0xFFFE)
#define SEQUENCE_END                    (0xFFFF)
#define SEQUENCE_FAST_SETMODE_START     (0xFFFD)
#define SEQUENCE_FAST_SETMODE_END       (0xFFFC)

#define I2C_ADDRESS						(0x78 >> 1)

/****************************************************************************
 * Implementing for a New Sensor:
 * 
 * When implementing for a new sensor from this template, there are four 
 * phases:
 * Phase 0: Gather sensor information. In phase 0, we should gather the sensor
 *          information for bringup such as sensor capabilities, set mode i2c
 *          write sequence, NvOdmIoAddress in nvodm_query_discovery_imager.h,
 *          and so on.
 * Phase 1: Bring up a sensor. After phase 1, we should be able to do still
 *          captures. Since this is a yuv sensor, there won't be many
 *          interactions between camera and sensor.
 * Phase 2: Fully functioning sensor. After phase 2, the sensor should be fully
 *          functioning as described in nvodm_imager.h
 *
 * To help implementing for a new sensor, the template code will be marked as
 * Phase 0, 1, 2
 ****************************************************************************/

/*
Callback Sequence:

Sensor_Open()
Sensor_GetCapabilities()
Sensor_ListModes()
Sensor_SetParameter(NvOdmImagerParameter_SensorInputClock)
Sensor_GetParameter(NvOdmImagerParameter_StereoCapable)
Sensor_GetParameter(NvOdmImagerParameter_SensorFrameRateLimits)
Sensor_GetParameter(NvOdmImagerParameter_GetBestSensorMode) NOTE: Return NV_FALSE to let it be handled by Camera Upper layers
Sensor_ListModes()
Sensor_SetParameter(NvOdmImagerParameter_OperationalMode)
Clock set [SENSOR]
Clock set [CORE]
Sensor_SetPowerLevel(NvOdmImagerPowerLevel_On)
Sensor_SetParameter(NvOdmImagerParameter_SensorInputClock)
Sensor_GetParameter(NvOdmImagerParameter_FlashCapabilities)
Sensor_GetParameter(NvOdmImagerParameter_TorchCapabilities)
Clock set [CORE] 24000 [External for Pads] [Internal Clock for Core] Result: 24000 KHz
Sensor_SetMode()

*/

/**
 * ----------------------------------------------------------
 *  Start of Phase 0 code
 * ----------------------------------------------------------
 */

/**
 * Sensor Specific Variables/Defines. Phase 0.
 *
 * If a new sensor code is created from this template, the following
 * variable/defines should be created for the new sensor specifically.
 * 1. A sensor capabilities (NvOdmImagerCapabilities) for this specific sensor
 * 2. A sensor set mode sequence list  (an array of SensorSetModeSequence)
 *    consisting of pairs of a sensor mode and corresponding set mode sequence
 * 3. A set mode sequence for each mode defined in 2.
 */

/**
 * Sensor yuv's capabilities. Phase 0. Sensor Dependent.
 * This is the first thing the camera driver requests. The information here
 * is used to setup the proper interface code (CSI, VIP, pixel type), start
 * up the clocks at the proper speeds, and so forth.
 * For Phase 0, one could start with these below as an example for a yuv
 * sensor and just update clocks from what is given in the reference documents
 * for that device. The FAE may have specified the clocks when giving the
 * reccommended i2c settings.
 */
static NvOdmImagerCapabilities g_SensorYuvCaps =
{
    "OV5642",  // string identifier, used for debugging
    NvOdmImagerSensorInterface_Parallel8, // interconnect type
    {
        NvOdmImagerPixelType_UYVY, //NvOdmImagerPixelType_UYVY,
    },
    NvOdmImagerOrientation_0_Degrees, // rotation with respect to main display
    NvOdmImagerDirection_Away,          // toward or away the main display
    24000, // initial sensor clock (in khz)
    { {24000, 48.0f/24.0f}, {0,0} }, // preferred clock profile
    {
        NvOdmImagerSyncEdge_Rising, // hsync polarity
        NvOdmImagerSyncEdge_Rising, // vsync polarity
		NV_TRUE,                   // uses mclk on vgp0
    }, // parallel settings (VIP)
    {0,0,0,NV_FALSE,0}, // serial settings (CSI) (not used)
    {0, 0}, // min blank time, shouldn't need adjustment
    0, // preferred mode, which resolution to use on startup
    0, // FOCUSER_GUID, only set if focuser code available
    0, // FLASH_GUID, only set if flash device code is available
    NVODM_IMAGER_CAPABILITIES_END
};

#include "sensor_yuv_ov5642_seqs.h"

/**
 * A list of sensor mode and its SetModeSequence. Phase 0. Sensor Dependent.
 * When SetMode is called, it searches the list for desired resolution and
 * frame rate and write the SetModeSequence through i2c.
 */
static SensorSetModeSequence g_SensorYuvSetModeSequenceList[] =
{
	// H,W muss 16 aligned sein
    // WxH				 x, y    fps    AR   PLL Mult     Set Mode Sequence
/*#if USE_QVGA_MODE
	{ { {  320,   240}, {0, 0},  15.00, 1.0, 48.0/24.0} , SetModeSequence_QVGA_YUV_15FPS	, NULL		},
#endif*/
#if USE_VGA_MODE
	{ { {  640,   480}, {0, 0},  15.00, 1.0, 48.0/24.0} , SetModeSequence_VGA_YUV_15FPS		, NULL		},
#endif
/*#if USE_SVGA_30_MODE
	{ { {  800,   600}, {0, 0},  30.00, 1.0, 96.0/24.0} , SetModeSequence_SVGA_YUV_30FPS		, NULL		},
#endif
#if USE_SXGA_30_MODE
	{ { { 1280,   960}, {0, 0},  30.00, 1.0, 96.0/24.0} , SetModeSequence_SXGA_YUV_30FPS		, NULL		},
#endif
#if USE_720P30_MODE
	{ { { 1280,   720}, {0, 0},  30.00, 1.0, 96.0/24.0} , SetModeSequence_720p_YUV_30FPS		, NULL		},
#endif
#if USE_1080P375_MODE
	{ { { 1920,  1080}, {0, 0},   3.75, 1.0, 48.0/24.0} , SetModeSequence_1080p_YUV_3_75FPS	, NULL	},
#endif
#if USE_1080P15_MODE
	{ { { 1920,  1080}, {0, 0},   15, 1.0, 96.0/24.0} , SetModeSequence_1080p_YUV_15FPS	, NULL	},
#endif
#if USE_UXGA_MODE
	{ { { 1600,  1200}, {0, 0},   7.50, 1.0, 144.0/24.0} , SetModeSequence_UXGA_YUV_7_5FPS	, NULL		},
#endif
#if	USE_1600_1200_MODE
	{ { { 1600,  1200}, {0, 0},   7.50, 1.0, 144.0/24.0} , SetModeSequence_1600_1200_YUV_7_5FPS	, NULL		},
#endif
#if USE_5MP_75_MODE
	{ { { 2592,  1944}, {0, 0},   7.50, 1.0, 144.0/24.0} , SetModeSequence_5MP_YUV_7_5FPS	, NULL		},
#endif
#if USE_5MP_375_MODE
	{ { { 2592,  1944}, {0, 0},   3.75, 1.0, 48.0/24.0} , SetModeSequence_5MP_YUV_3_75FPS	, NULL		},
#endif*/
};

/**
 * ----------------------------------------------------------
 *  Start of Phase 1 code
 * ----------------------------------------------------------
 */

NvBool WriteToI2C(DevCtrlReg16* SetModeSequence)
{
	DevCtrlReg16 *ptr = SetModeSequence;
	NvBool ret= TRUE;

	BYTE payload[3];

	DWORD VerMaj;
	DWORD VerMin;
	DWORD VerBuild;

	I2cIntGetLibVersion(&VerMaj, &VerMin, &VerBuild);
	DEBUGMSG(ZONE_FUNC, (_T("+WriteToI2C, I2C Library Version %d.%d Build %d\n"), VerMaj, VerMin, VerBuild));

	if (I2CInit() /*I2CInitEx(0x7000C500)*/)
//	if (I2CInitEx(0x7000C500)) // I2C_3
	{
		if(GetI2CLock(500)){
			I2CSetSpeed(I2C_100KBPS);				// Currently, this sets LastError=1210 on Tegra
			
			while(ptr->RegAddr!= SEQUENCE_END)
			{
				if(ptr->RegAddr == SEQUENCE_WAIT_MS){
					Sleep(ptr->RegValue);
				}
				else{

					payload[0] = (ptr->RegAddr) >> 8;		// need to send high address byte first
					payload[1] = (ptr->RegAddr) & 0xFF;
					payload[2] = (ptr->RegValue) & 0xFF;

					if(!I2CBurstWrite(I2C_ADDRESS, (BYTE*) &payload, -1, 3))
					{
						DEBUGMSG(ZONE_ERR, (L"WriteToI2C I2CBurstWrite failed\n"));
						ret= FALSE;
						break;
					}
				}
				ptr += 1;
			}
			ReleaseI2CLock();
		}
		else{
			ret= FALSE;
			DEBUGMSG(ZONE_ERR, (L"WriteToI2C GetI2CLock failed\n"));
		}
		I2CDeInit();
	}
	else{
		ret= FALSE;
		DEBUGMSG(ZONE_FUNC, (L"WriteToI2C I2CInit failed\n"));
	}
	DEBUGMSG(ZONE_FUNC, (L"-WriteToI2C\n"));
	return ret;
}

static BOOL SensorSetup()
{
//	BOOL Success;

	DEBUGMSG(ZONE_FUNC, (L"+SensorSetup\n"));

	InitGPIOLib();

	// PINs fr OV5642 setzen
	SetPinAltFn(CAM_PWRDWN_PIN, -1, DIR_OUT);
	SetPinAltFn(CAM_RESET_PIN, -1, DIR_OUT);

	// AltFn fuer die anderen beiden PIN's setzen
	SetPinAltFn(166, -1, DIR_OUT);
	SetPinAltFn(172, -1, DIR_OUT);

	SetPinLevel(CAM_PWRDWN_PIN, ~CAM_PWRDWN_POL);
	Sleep(150);
	SetPinLevel(CAM_PWRDWN_PIN, CAM_PWRDWN_POL);

	SetPinLevel(CAM_RESET_PIN, ~CAM_RESET_POL);
	Sleep(150);
	SetPinLevel(CAM_RESET_PIN, CAM_RESET_POL);

	DeInitGPIOLib();

	DEBUGMSG(ZONE_FUNC, (L"-SensorSetup\n"));

	return 1 ;
}


/**
 * Sensor yuv's private context. Phase 1.
 */
typedef struct SensorYuvContextRec
{
    // Phase 1 variables.
    //NvOdmImagerI2cConnection I2c;
    NvU32 ModeIndex;
    NvU32 NumModes;
    NvOdmImagerPowerLevel PowerLevel;
    NvF32 FrameRate;
    //ImagerGpioConfig GpioConfig;

}SensorYuvContext;

/**
 * Static functions
 * Some compilers require these to be declared upfront.
 */
static NvBool SensorYuv_Open(NvOdmImagerHandle hImager);
static void SensorYuv_Close(NvOdmImagerHandle hImager);

static void
SensorYuv_GetCapabilities(
    NvOdmImagerHandle hImager,
    NvOdmImagerCapabilities *pCapabilities);

static void
SensorYuv_ListModes(
    NvOdmImagerHandle hImager,
    NvOdmImagerSensorMode *pModes,
    NvS32 *pNumberOfModes);
    
static NvBool
SensorYuv_SetMode(
    NvOdmImagerHandle hImager,
    const SetModeParameters *pParameters,
    NvOdmImagerSensorMode *pSelectedMode,
    SetModeParameters *pResult);

static NvBool
SensorYuv_SetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel PowerLevel);

static NvBool
SensorYuv_SetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    const void *pValue);

static NvBool
SensorYuv_GetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    void *pValue);

static void
SensorYuv_GetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel *pPowerLevel);


/**
 * SensorYuv_Open. Phase 1.
 * Initialize sensor yuv and its private context
 */
static NvBool SensorYuv_Open(NvOdmImagerHandle hImager)
{
    SensorYuvContext *pSensorYuvContext = NULL;
	NvBool Status = NV_FALSE;

	DEBUGMSG(ZONE_FUNC, (_T("+SensorYuv_Open\n")));

	if (!hImager || !hImager->pSensor) {
		DEBUGMSG(ZONE_FUNC, (_T("-SensorYuv_Open !hImager\n")));
        return NV_FALSE;
	}

    pSensorYuvContext =
        (SensorYuvContext*)malloc(sizeof(SensorYuvContext));
    if (!pSensorYuvContext)
        goto fail;

    memset(pSensorYuvContext, 0, sizeof(SensorYuvContext));

	//Set GPIOs (for the moment just set the multiplexed ones the main pins are setup by the driver
	//for verfication you can use the GPIOconfig Tool)
	SetGPIOAltFn(TEGRA_GPIONUM('b',4),-1,DIR_IN);	//D7 Sodimm 59
	SetGPIOAltFn(TEGRA_GPIONUM('a',7),-1,DIR_IN);	//D6 Sodimm 67

    pSensorYuvContext->NumModes = NV_ARRAY_SIZE(g_SensorYuvSetModeSequenceList);
    pSensorYuvContext->ModeIndex = pSensorYuvContext->NumModes + 1; // invalid mode
    
    hImager->pSensor->pPrivateContext = pSensorYuvContext;

	//DEBUGMSG(1, (_T("+SensorSetup\n")));
	//SensorSetup();
	//DEBUGMSG(1, (_T("-SensorSetup\n")));

	DEBUGMSG(ZONE_FUNC, (_T("-SensorYuv_Open OK\n")));
    return NV_TRUE;    

fail:
    free(pSensorYuvContext);
    return NV_FALSE;
}

/**
 * SensorYuv_Close. Phase 1.
 * Free the sensor's private context
 */
static void SensorYuv_Close(NvOdmImagerHandle hImager)
{
    SensorYuvContext *pContext = NULL;
        
    if (!hImager || !hImager->pSensor || !hImager->pSensor->pPrivateContext)
        return;

    pContext = (SensorYuvContext*)hImager->pSensor->pPrivateContext;

    // cleanup
    free(pContext);
    hImager->pSensor->pPrivateContext = NULL;
}

/**
 * SensorYuv_GetCapabilities. Phase 1.
 * Returnt sensor Yuv's capabilities
 */
static void
SensorYuv_GetCapabilities(
    NvOdmImagerHandle hImager,
    NvOdmImagerCapabilities *pCapabilities)
{
	DEBUGMSG(ZONE_FUNC, (L"+SensorYuv_GetCapabilities\n"));
    // Copy the sensor caps from g_SensorYuvCaps
    memcpy(pCapabilities, &g_SensorYuvCaps,
        sizeof(NvOdmImagerCapabilities));

	DEBUGMSG(ZONE_FUNC, (L"-SensorYuv_GetCapabilities\n"));
}

/**
 * SensorYuv_ListModes. Phase 1.
 * Return a list of modes that sensor yuv supports.
 * If called with a NULL ptr to pModes, then it just returns the count
 */
static void
SensorYuv_ListModes(
    NvOdmImagerHandle hImager,
    NvOdmImagerSensorMode *pModes,
    NvS32 *pNumberOfModes)
{
    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;
    NvS32 i;

	DEBUGMSG(ZONE_FUNC, (L"+SensorYuv_ListModes\n"));

    if (pNumberOfModes)
    {
        *pNumberOfModes = (NvS32)pContext->NumModes;
        if (pModes)
        {
            // Copy the modes from g_SensorYuvSetModeSequenceList
            for (i = 0; i < *pNumberOfModes; i++)
                pModes[i] = g_SensorYuvSetModeSequenceList[i].Mode;
        }
    }

	DEBUGMSG(ZONE_FUNC, (L"-SensorYuv_ListModes\n"));
}

/**
 * SensorYuv_SetMode. Phase 1.
 * Set sensor yuv to the mode of the desired resolution and frame rate.
 */
static NvBool
SensorYuv_SetMode(
    NvOdmImagerHandle hImager,
    const SetModeParameters *pParameters,
    NvOdmImagerSensorMode *pSelectedMode,
    SetModeParameters *pResult)
{
    NvBool Status = NV_FALSE;

    SensorYuvContext *pContext = (SensorYuvContext*)hImager->pSensor->pPrivateContext;

    NvU32 Index;

	DEBUGMSG(ZONE_FUNC, (_T("+SensorYuv_SetMode %dx%d\n"), pParameters->Resolution.width, pParameters->Resolution.height));

    //ASSERT(pParameters);

    //NKDbgPrintfW("Setting resolution to %dx%d\n",
    //    pParameters->Resolution.width, pParameters->Resolution.height);

    // Find the right entrys in g_SensorYuvSetModeSequenceList that matches
    // the desired resolution and framerate
    for (Index = 0; Index < pContext->NumModes; Index++)
    {
        if ((pParameters->Resolution.width == g_SensorYuvSetModeSequenceList[Index].Mode.ActiveDimensions.width) &&
            (pParameters->Resolution.height == g_SensorYuvSetModeSequenceList[Index].Mode.ActiveDimensions.height) /*&&
            (pContext->FrameRate == g_SensorYuvSetModeSequenceList[Index].Mode.PeakFrameRate)*/ )
            break;
    }

    // No match found
	if (Index == pContext->NumModes) {
		DEBUGMSG(ZONE_ERR, (_T("-SensorYuv_SetMode no match found\n")));
        return NV_FALSE;
	}

    if (pSelectedMode)
    {
		DEBUGMSG(ZONE_INFO, (_T("Setting resolution: %ix%i with %0.2f fps.\n"), g_SensorYuvSetModeSequenceList[Index].Mode.ActiveDimensions.width, g_SensorYuvSetModeSequenceList[Index].Mode.ActiveDimensions.height, g_SensorYuvSetModeSequenceList[Index].Mode.PeakFrameRate));
        *pSelectedMode = g_SensorYuvSetModeSequenceList[Index].Mode;
    }

	if (pContext->ModeIndex == Index) {
		DEBUGMSG(ZONE_FUNC, (_T("-SensorYuv_SetMode same mode\n")));
		return NV_TRUE;
	}

	//if(!SensorSetup()) {
	//	DEBUGMSG(1, (_T("-SensorYuv_SetMode OV5642 SetupUp mode change failed\n")));
	//	return NV_FALSE;
	//}
	
	Sleep(250);

    // i2c writes for the set mode sequence
	DEBUGMSG(ZONE_INFO, (_T(" SensorYuv_SetMode %d Write ModeSequence\n"), Index));
    Status = WriteToI2C(g_SensorYuvSetModeSequenceList[Index].pSequence);

	if (!Status) {
		DEBUGMSG(ZONE_FUNC, (_T("-SensorYuv_SetMode OV5642 I2C failed\n")));
       return NV_FALSE;
	}

    pContext->ModeIndex = Index;

    if (pResult)
    {
        pResult->Resolution = g_SensorYuvSetModeSequenceList[Index].
                              Mode.ActiveDimensions;
        pResult->Exposure = 0;
        memset(pResult->Gains, 0, sizeof(NvF32) * 4);
    }

	DEBUGMSG(ZONE_FUNC, (_T("-SensorYuv_SetMode done\n")));
    return Status;
}

/**
 * SensorYuv_SetPowerLevel. Phase 1
 * Set the sensor's power level
 */
static NvBool
SensorYuv_SetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel PowerLevel)
{
    NvBool Status = NV_TRUE;
    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;
    //const NvOdmPeripheralConnectivity *pConnections;
    const char* powerlevel_str[6] = {"?0", "Off","StandBy", "On", "SyncSensors"};

	DEBUGMSG(ZONE_FUNC, (_T("+SensorYuv_SetPowerLevel %S\n"), powerlevel_str[PowerLevel]));

    if (pContext->PowerLevel == PowerLevel)
        goto exit;

    ////NV_ASSERT(hImager->pSensor->pConnections);
    //pConnections = hImager->pSensor->pConnections;

    switch(PowerLevel)
    {
        case NvOdmImagerPowerLevel_On:

        SensorSetup();
/* mcr: setting initial resolution of 320x240 is not neccessary.
				// i2c writes for the set mode sequence
				DEBUGMSG(1, (_T("+Write ModeSequence\n")));
				printf("Setting resolution: %ix%i with %0.2f fps.\n", g_SensorYuvSetModeSequenceList[0].Mode.ActiveDimensions.width, g_SensorYuvSetModeSequenceList[0].Mode.ActiveDimensions.height, g_SensorYuvSetModeSequenceList[0].Mode.PeakFrameRate);
				Status = WriteToI2C(g_SensorYuvSetModeSequenceList[0].pSequence);

				if (!Status) {
					DEBUGMSG(1, (_T("-Write ModeSequence failed\n")));
				} else {
					DEBUGMSG(1, (_T("-Write ModeSequence OK\n")));
				}
*/

/*
            Status = SetPowerLevelWithPeripheralConnectivity(pConnections,
                         &pContext->I2c, &pContext->GpioConfig,
                         NvOdmImagerPowerLevel_On);
            if (!Status)
            {
                NvOdmOsDebugPrintf("NvOdmImagerPowerLevel_On failed.\n");
                return NV_FALSE;
            }
*/
            break;

        case NvOdmImagerPowerLevel_Standby:
/*
            Status = SetPowerLevelWithPeripheralConnectivity(pConnections,
                         &pContext->I2c, &pContext->GpioConfig,
                         NvOdmImagerPowerLevel_Standby);
*/
            break;
        case NvOdmImagerPowerLevel_Off:
/*
            Status = SetPowerLevelWithPeripheralConnectivity(pConnections,
                         &pContext->I2c, &pContext->GpioConfig,
                         NvOdmImagerPowerLevel_Off);
            
            // wait for the standby 0.1 sec should be enough
            NvOdmOsWaitUS(100000); 
*/
            break;
        default:
//            //NV_ASSERT("!Unknown power level\n");
            Status = NV_FALSE;
            break;
    }

    if (Status)
        pContext->PowerLevel = PowerLevel;

exit:
	DEBUGMSG(ZONE_FUNC, (L"-SensorYuv_SetPowerLevel returns %s\n", Status?L"True":L"False"));

    return Status;
}




/**
 * SensorYuv_SetParameter. Phase 2.
 * Set sensor specific parameters. 
 * For Phase 1. This can return NV_TRUE.
 */
static NvBool
SensorYuv_SetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    const void *pValue)
{
    if(Param==NvOdmImagerParameter_SensorTriggerStill)
        return NV_TRUE;

    DEBUGMSG(ZONE_FUNC, (L"+SensorYuv_SetParameter(%S)\n", GetParamName(Param)));
    
    switch(Param)
    {
        case NvOdmImagerParameter_SensorInputClock:

            if(SizeOfValue == sizeof(NvOdmImagerClockProfile))
            {
                // Verify the input clock is valid.
                NvU32 InClock =
                    ((NvOdmImagerClockProfile*)pValue)->ExternalClockKHz;

                DEBUGMSG(ZONE_FUNC && ZONE_INFO, (L"Clock=%d KHz\r\n", InClock));
				
				if (!InClock) {
					return NV_FALSE;
				}
                //if (InClock < g_SensorYuvCaps.InitialSensorClockRateKHz)
                //    return NV_FALSE;
            }
            break;

        case NvOdmImagerParameter_OptimizeResolutionChange:
			if(SizeOfValue!=sizeof(NvBool)){break;}

            // Sensor yuv doesn't support optimized resolution change.
            //if (*((NvBool*)pValue) == NV_FALSE)
			if (*((NvBool*)pValue) == NV_TRUE)
                return NV_TRUE;
            else
                return NV_FALSE;

            break;

        case NvOdmImagerParameter_SelfTest:
            // not implemented.
            break;
        default:
//            //NV_ASSERT(!"Not Implemented");
            break;
    }


	DEBUGMSG(ZONE_FUNC, (L"-SensorYuv_SetParameter\n"));

    return NV_TRUE;
}

/**
 * SensorYuv_GetParameter. Phase 1.
 * Get sensor specific parameters
 */
static NvBool
SensorYuv_GetParameter(
    NvOdmImagerHandle hImager,
    NvOdmImagerParameter Param,
    NvS32 SizeOfValue,
    void *pValue)
{
    NvBool result = NV_TRUE;

    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;

    DEBUGMSG(ZONE_FUNC, (L"+SensorYuv_GetParameter(%S)\n", GetParamName(Param)));

    switch(Param)
    {
        case NvOdmImagerParameter_DeviceStatus:
            if(SizeOfValue ==sizeof(NvOdmImagerDeviceStatus))
            {
                NvOdmImagerDeviceStatus *pStatus;
                pStatus = (NvOdmImagerDeviceStatus *)pValue;
/*
                {
                    NvOdmImagerI2cConnection *pI2c = &pContext->I2c;
                    // Pick your own useful registers to use for debug.
                    // If the camera hangs, these register values are printed
                    // Sensor Dependent.
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x0000);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[0]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2104);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[1]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2703);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[2]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2705);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[3]);
                    NvOdmImagerI2cWrite(pI2c, 0x098C, 0x2737);
                    NvOdmImagerI2cRead(pI2c, 0x0990, &pStatus->Values[4]);

                    pStatus->Count = 5;
                }
*/
            }
            break;

        case NvOdmImagerParameter_SensorFrameRate:
			if(SizeOfValue == sizeof(NvF32)){            
                *((NvF32*)pValue) = g_SensorYuvSetModeSequenceList[pContext->ModeIndex].Mode.PeakFrameRate;
                break;
			}

        case NvOdmImagerParameter_GetBestSensorMode:
            if(SizeOfValue == sizeof(NvOdmImagerSensorMode))
            {
                NvOdmImagerSensorMode* pMode = (NvOdmImagerSensorMode *) pValue;
                DEBUGMSG(ZONE_INFO, (L"ModeToSet: %dx%d (%d fps)\r\n", pMode->ActiveDimensions.width, pMode->ActiveDimensions.height, (NvU32)pMode->PeakFrameRate));
                pContext->FrameRate = pMode->PeakFrameRate;
                //return NV_TRUE;
                result = NV_FALSE;
                break;
            }

        case NvOdmImagerParameter_ISPSetting:
            if (pValue &&
                SizeOfValue == sizeof(NvOdmImagerISPSetting))
            {
                NvOdmImagerISPSetting *pSetting =
                    (NvOdmImagerISPSetting *)pValue;

                DEBUGMSG(ZONE_INFO, (L"ISP Setting: 0x%x %d\r\n",pSetting->attribute, pSetting->dataSize));

                switch(pSetting->attribute)
                {
                case NvCameraIspAttribute_FlickerFrequency:
                    break;
                }
            }
            break;

        default:
           //NV_ASSERT(!"Not Implemented");
           DEBUGMSG(ZONE_FUNC && ZONE_WARN, (L"Not Implemented\n"));
           result = NV_FALSE;
           break;
    }

	DEBUGMSG(ZONE_FUNC, (L"-SensorYuv_GetParameter\n"));

    return result;
}

/**
 * SensorYuv_GetPowerLevel. Phase 2.
 * Get the sensor's current power level
 */
static void
SensorYuv_GetPowerLevel(
    NvOdmImagerHandle hImager,
    NvOdmImagerPowerLevel *pPowerLevel)
{
	

    SensorYuvContext *pContext =
        (SensorYuvContext*)hImager->pSensor->pPrivateContext;


	DEBUGMSG(ZONE_FUNC, (L"+SensorYuv_GetPowerLevel\n"));

    *pPowerLevel = pContext->PowerLevel;

	DEBUGMSG(ZONE_FUNC, (L"-SensorYuv_GetPowerLevel\n"));
}

/**
 * SensorYuv_GetHal. Phase 1.
 * return the hal functions associated with sensor yuv
 */
NvBool SensorPlugin_GetHal_YUV_OV5642(NvOdmImagerHandle hImager)
{
    if (!hImager || !hImager->pSensor)
        return NV_FALSE;

    hImager->pSensor->pfnOpen = SensorYuv_Open;
    hImager->pSensor->pfnClose = SensorYuv_Close;
    hImager->pSensor->pfnGetCapabilities = SensorYuv_GetCapabilities;
    hImager->pSensor->pfnListModes = SensorYuv_ListModes;
    hImager->pSensor->pfnSetMode = SensorYuv_SetMode;
    hImager->pSensor->pfnSetPowerLevel = SensorYuv_SetPowerLevel;
    hImager->pSensor->pfnGetPowerLevel = SensorYuv_GetPowerLevel;
    hImager->pSensor->pfnSetParameter = SensorYuv_SetParameter;
    hImager->pSensor->pfnGetParameter = SensorYuv_GetParameter;

    return NV_TRUE;
}
